/*
 * tk.h --
 *
 *	Declarations for Tk-related things that are visible
 *	outside of the Tk module itself.
 *
 * Copyright (c) 1989-1994 The Regents of the University of California.
 * Copyright (c) 1994 The Australian National University.
 * Copyright (c) 1994-1997 Sun Microsystems, Inc.
 * Copyright (c) 1998 by Scriptics Corporation.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: tk.h,v 1.17 1998/10/16 17:22:59 escoffon Exp $
 */

#ifndef _TK
#define _TK

/*
 * When version numbers change here, you must also go into the following files
 * and update the version numbers:
 *
 * README
 * unix/configure.in
 * win/makefile.bc	(Not for patch release updates)
 * win/makefile.vc	(Not for patch release updates)
 * win/README
 * library/tk.tcl
 *
 * The release level should be  0 for alpha, 1 for beta, and 2 for
 * final/patch.  The release serial value is the number that follows the
 * "a", "b", or "p" in the patch level; for example, if the patch level
 * is 4.3b2, TK_RELEASE_SERIAL is 2.  It restarts at 1 whenever the
 * release level is changed, except for the final release, which should
 * be 0.
 *
 * You may also need to update some of these files when the numbers change
 * for the version of Tcl that this release of Tk is compiled against.
 */

#define TK_MAJOR_VERSION   8
#define TK_MINOR_VERSION   0
#define TK_RELEASE_LEVEL   2
#define TK_RELEASE_SERIAL  4

#define TK_VERSION "8.0"
#define TK_PATCH_LEVEL "8.0.4+"

/* 
 * A special definition used to allow this header file to be included 
 * in resource files.
 */

#ifndef RESOURCE_INCLUDED

/*
 * The following definitions set up the proper options for Macintosh
 * compilers.  We use this method because there is no autoconf equivalent.
 */

#ifdef MAC_TCL
#   ifndef REDO_KEYSYM_LOOKUP
#	define REDO_KEYSYM_LOOKUP
#   endif
#endif

#ifndef _TCL
#   include <tcl.h>
#endif
#ifndef _XLIB_H
#   ifdef MAC_TCL
#	include <Xlib.h>
#	include <X.h>
#   else
#	include <X11/Xlib.h>
#   endif
#endif
#ifdef __STDC__
#   include <stddef.h>
#endif

#ifdef BUILD_tk
# undef TCL_STORAGE_CLASS
# define TCL_STORAGE_CLASS DLLEXPORT
#endif

/*
 * Decide whether or not to use input methods.
 */

#ifdef XNQueryInputStyle
#define TK_USE_INPUT_METHODS
#endif

/*
 * Dummy types that are used by clients:
 */

typedef struct Tk_BindingTable_ *Tk_BindingTable;
typedef struct Tk_Canvas_ *Tk_Canvas;
typedef struct Tk_Cursor_ *Tk_Cursor;
typedef struct Tk_ErrorHandler_ *Tk_ErrorHandler;
typedef struct Tk_Font_ *Tk_Font;
typedef struct Tk_Image__ *Tk_Image;
typedef struct Tk_ImageMaster_ *Tk_ImageMaster;
typedef struct Tk_PostscriptInfo_ *Tk_PostscriptInfo;
typedef struct Tk_TextLayout_ *Tk_TextLayout;
typedef struct Tk_Window_ *Tk_Window;
typedef struct Tk_3DBorder_ *Tk_3DBorder;

/*
 * Additional types exported to clients.
 */

typedef char *Tk_Uid;

/*
 * Structure used to specify how to handle argv options.
 */

typedef struct {
    char *key;		/* The key string that flags the option in the
			 * argv array. */
    int type;		/* Indicates option type;  see below. */
    char *src;		/* Value to be used in setting dst;  usage
			 * depends on type. */
    char *dst;		/* Address of value to be modified;  usage
			 * depends on type. */
    char *help;		/* Documentation message describing this option. */
} Tk_ArgvInfo;

/*
 * Legal values for the type field of a Tk_ArgvInfo: see the user
 * documentation for details.
 */

#define TK_ARGV_CONSTANT		15
#define TK_ARGV_INT			16
#define TK_ARGV_STRING			17
#define TK_ARGV_UID			18
#define TK_ARGV_REST			19
#define TK_ARGV_FLOAT			20
#define TK_ARGV_FUNC			21
#define TK_ARGV_GENFUNC			22
#define TK_ARGV_HELP			23
#define TK_ARGV_CONST_OPTION		24
#define TK_ARGV_OPTION_VALUE		25
#define TK_ARGV_OPTION_NAME_VALUE	26
#define TK_ARGV_END			27

/*
 * Flag bits for passing to Tk_ParseArgv:
 */

#define TK_ARGV_NO_DEFAULTS		0x1
#define TK_ARGV_NO_LEFTOVERS		0x2
#define TK_ARGV_NO_ABBREV		0x4
#define TK_ARGV_DONT_SKIP_FIRST_ARG	0x8

/*
 * Structure used to describe application-specific configuration
 * options:  indicates procedures to call to parse an option and
 * to return a text string describing an option.
 */

typedef int (Tk_OptionParseProc) _ANSI_ARGS_((ClientData clientData,
	Tcl_Interp *interp, Tk_Window tkwin, char *value, char *widgRec,
	int offset));
typedef char *(Tk_OptionPrintProc) _ANSI_ARGS_((ClientData clientData,
	Tk_Window tkwin, char *widgRec, int offset,
	Tcl_FreeProc **freeProcPtr));

typedef struct Tk_CustomOption {
    Tk_OptionParseProc *parseProc;	/* Procedure to call to parse an
					 * option and store it in converted
					 * form. */
    Tk_OptionPrintProc *printProc;	/* Procedure to return a printable
					 * string describing an existing
					 * option. */
    ClientData clientData;		/* Arbitrary one-word value used by
					 * option parser:  passed to
					 * parseProc and printProc. */
} Tk_CustomOption;

/*
 * Structure used to specify information for Tk_ConfigureWidget.  Each
 * structure gives complete information for one option, including
 * how the option is specified on the command line, where it appears
 * in the option database, etc.
 */

typedef struct Tk_ConfigSpec {
    int type;			/* Type of option, such as TK_CONFIG_COLOR;
				 * see definitions below.  Last option in
				 * table must have type TK_CONFIG_END. */
    char *argvName;		/* Switch used to specify option in argv.
				 * NULL means this spec is part of a group. */
    char *dbName;		/* Name for option in option database. */
    char *dbClass;		/* Class for option in database. */
    char *defValue;		/* Default value for option if not
				 * specified in command line or database. */
    int offset;			/* Where in widget record to store value;
				 * use Tk_Offset macro to generate values
				 * for this. */
    int specFlags;		/* Any combination of the values defined
				 * below;  other bits are used internally
				 * by tkConfig.c. */
    Tk_CustomOption *customPtr;	/* If type is TK_CONFIG_CUSTOM then this is
				 * a pointer to info about how to parse and
				 * print the option.  Otherwise it is
				 * irrelevant. */
} Tk_ConfigSpec;

/*
 * Type values for Tk_ConfigSpec structures.  See the user
 * documentation for details.
 */

#define TK_CONFIG_BOOLEAN	1
#define TK_CONFIG_INT		2
#define TK_CONFIG_DOUBLE	3
#define TK_CONFIG_STRING	4
#define TK_CONFIG_UID		5
#define TK_CONFIG_COLOR		6
#define TK_CONFIG_FONT		7
#define TK_CONFIG_BITMAP	8
#define TK_CONFIG_BORDER	9
#define TK_CONFIG_RELIEF	10
#define TK_CONFIG_CURSOR	11
#define TK_CONFIG_ACTIVE_CURSOR	12
#define TK_CONFIG_JUSTIFY	13
#define TK_CONFIG_ANCHOR	14
#define TK_CONFIG_SYNONYM	15
#define TK_CONFIG_CAP_STYLE	16
#define TK_CONFIG_JOIN_STYLE	17
#define TK_CONFIG_PIXELS	18
#define TK_CONFIG_MM		19
#define TK_CONFIG_WINDOW	20
#define TK_CONFIG_CUSTOM	21
#define TK_CONFIG_END		22

/*
 * Macro to use to fill in "offset" fields of Tk_ConfigInfos.
 * Computes number of bytes from beginning of structure to a
 * given field.
 */

#ifdef offsetof
#define Tk_Offset(type, field) ((int) offsetof(type, field))
#else
#define Tk_Offset(type, field) ((int) ((char *) &((type *) 0)->field))
#endif

/*
 * Possible values for flags argument to Tk_ConfigureWidget:
 */

#define TK_CONFIG_ARGV_ONLY	1
#define TK_CONFIG_OBJS		0x80

/*
 * Possible flag values for Tk_ConfigInfo structures.  Any bits at
 * or above TK_CONFIG_USER_BIT may be used by clients for selecting
 * certain entries.  Before changing any values here, coordinate with
 * tkConfig.c (internal-use-only flags are defined there).
 */

#define TK_CONFIG_COLOR_ONLY		1
#define TK_CONFIG_MONO_ONLY		2
#define TK_CONFIG_NULL_OK		4
#define TK_CONFIG_DONT_SET_DEFAULT	8
#define TK_CONFIG_OPTION_SPECIFIED	0x10
#define TK_CONFIG_USER_BIT		0x100

/*
 * Enumerated type for describing actions to be taken in response
 * to a restrictProc established by Tk_RestrictEvents.
 */

typedef enum {
    TK_DEFER_EVENT, TK_PROCESS_EVENT, TK_DISCARD_EVENT
} Tk_RestrictAction;

/*
 * Priority levels to pass to Tk_AddOption:
 */

#define TK_WIDGET_DEFAULT_PRIO	20
#define TK_STARTUP_FILE_PRIO	40
#define TK_USER_DEFAULT_PRIO	60
#define TK_INTERACTIVE_PRIO	80
#define TK_MAX_PRIO		100

/*
 * Relief values returned by Tk_GetRelief:
 */

#define TK_RELIEF_RAISED	1
#define TK_RELIEF_FLAT		2
#define TK_RELIEF_SUNKEN	4
#define TK_RELIEF_GROOVE	8
#define TK_RELIEF_RIDGE		16
#define TK_RELIEF_SOLID		32

/*
 * "Which" argument values for Tk_3DBorderGC:
 */

#define TK_3D_FLAT_GC		1
#define TK_3D_LIGHT_GC		2
#define TK_3D_DARK_GC		3

/*
 * Special EnterNotify/LeaveNotify "mode" for use in events
 * generated by tkShare.c.  Pick a high enough value that it's
 * unlikely to conflict with existing values (like NotifyNormal)
 * or any new values defined in the future.
 */

#define TK_NOTIFY_SHARE		20

/*
 * Enumerated type for describing a point by which to anchor something:
 */

typedef enum {
    TK_ANCHOR_N, TK_ANCHOR_NE, TK_ANCHOR_E, TK_ANCHOR_SE,
    TK_ANCHOR_S, TK_ANCHOR_SW, TK_ANCHOR_W, TK_ANCHOR_NW,
    TK_ANCHOR_CENTER
} Tk_Anchor;

/*
 * Enumerated type for describing a style of justification:
 */

typedef enum {
    TK_JUSTIFY_LEFT, TK_JUSTIFY_RIGHT, TK_JUSTIFY_CENTER
} Tk_Justify;

/*
 * The following structure is used by Tk_GetFontMetrics() to return
 * information about the properties of a Tk_Font.  
 */

typedef struct Tk_FontMetrics {
    int ascent;			/* The amount in pixels that the tallest
				 * letter sticks up above the baseline, plus
				 * any extra blank space added by the designer
				 * of the font. */
    int descent;		/* The largest amount in pixels that any
				 * letter sticks below the baseline, plus any
				 * extra blank space added by the designer of
				 * the font. */
    int linespace;		/* The sum of the ascent and descent.  How
				 * far apart two lines of text in the same
				 * font should be placed so that none of the
				 * characters in one line overlap any of the
				 * characters in the other line. */
} Tk_FontMetrics;

/*
 * Flags passed to Tk_MeasureChars:
 */

#define TK_WHOLE_WORDS		1
#define TK_AT_LEAST_ONE		2
#define TK_PARTIAL_OK		4

/*
 * Flags passed to Tk_ComputeTextLayout:
 */

#define TK_IGNORE_TABS		8
#define TK_IGNORE_NEWLINES	16

/*
 * Each geometry manager (the packer, the placer, etc.) is represented
 * by a structure of the following form, which indicates procedures
 * to invoke in the geometry manager to carry out certain functions.
 */

typedef void (Tk_GeomRequestProc) _ANSI_ARGS_((ClientData clientData,
	Tk_Window tkwin));
typedef void (Tk_GeomLostSlaveProc) _ANSI_ARGS_((ClientData clientData,
	Tk_Window tkwin));

typedef struct Tk_GeomMgr {
    char *name;			/* Name of the geometry manager (command
				 * used to invoke it, or name of widget
				 * class that allows embedded widgets). */
    Tk_GeomRequestProc *requestProc;
				/* Procedure to invoke when a slave's
				 * requested geometry changes. */
    Tk_GeomLostSlaveProc *lostSlaveProc;
				/* Procedure to invoke when a slave is
				 * taken away from one geometry manager
				 * by another.  NULL means geometry manager
				 * doesn't care when slaves are lost. */
} Tk_GeomMgr;

/*
 * Result values returned by Tk_GetScrollInfo:
 */

#define TK_SCROLL_MOVETO	1
#define TK_SCROLL_PAGES		2
#define TK_SCROLL_UNITS		3
#define TK_SCROLL_ERROR		4

/*
 *---------------------------------------------------------------------------
 *
 * Extensions to the X event set
 *
 *---------------------------------------------------------------------------
 */
#define VirtualEvent	    (LASTEvent)
#define ActivateNotify	    (LASTEvent + 1)
#define DeactivateNotify    (LASTEvent + 2)
#define MouseWheelEvent     (LASTEvent + 3)
#define TK_LASTEVENT	    (LASTEvent + 4)

#define MouseWheelMask	    (1L << 28)

#define ActivateMask	    (1L << 29)
#define VirtualEventMask    (1L << 30)
#define TK_LASTEVENT	    (LASTEvent + 4)


/*
 * A virtual event shares most of its fields with the XKeyEvent and
 * XButtonEvent structures.  99% of the time a virtual event will be
 * an abstraction of a key or button event, so this structure provides
 * the most information to the user.  The only difference is the changing
 * of the detail field for a virtual event so that it holds the name of the
 * virtual event being triggered.
 */

typedef struct {
    int type;
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* True if this came from a SendEvent request */
    Display *display;	    /* Display the event was read from */
    Window event;	    /* Window on which event was requested. */
    Window root;	    /* root window that the event occured on */
    Window subwindow;	    /* child window */
    Time time;		    /* milliseconds */
    int x, y;		    /* pointer x, y coordinates in event window */
    int x_root, y_root;	    /* coordinates relative to root */
    unsigned int state;	    /* key or button mask */
    Tk_Uid name;	    /* Name of virtual event. */
    Bool same_screen;	    /* same screen flag */
} XVirtualEvent;

typedef struct {
    int type;
    unsigned long serial;   /* # of last request processed by server */
    Bool send_event;	    /* True if this came from a SendEvent request */
    Display *display;	    /* Display the event was read from */
    Window window;	    /* Window in which event occurred. */
} XActivateDeactivateEvent;
typedef XActivateDeactivateEvent XActivateEvent;
typedef XActivateDeactivateEvent XDeactivateEvent;

/*
 *--------------------------------------------------------------
 *
 * Macros for querying Tk_Window structures.  See the
 * manual entries for documentation.
 *
 *--------------------------------------------------------------
 */

#define Tk_Display(tkwin)		(((Tk_FakeWin *) (tkwin))->display)
#define Tk_ScreenNumber(tkwin)		(((Tk_FakeWin *) (tkwin))->screenNum)
#define Tk_Screen(tkwin)		(ScreenOfDisplay(Tk_Display(tkwin), \
	Tk_ScreenNumber(tkwin)))
#define Tk_Depth(tkwin)			(((Tk_FakeWin *) (tkwin))->depth)
#define Tk_Visual(tkwin)		(((Tk_FakeWin *) (tkwin))->visual)
#define Tk_WindowId(tkwin)		(((Tk_FakeWin *) (tkwin))->window)
#define Tk_PathName(tkwin) 		(((Tk_FakeWin *) (tkwin))->pathName)
#define Tk_Name(tkwin)			(((Tk_FakeWin *) (tkwin))->nameUid)
#define Tk_Class(tkwin) 		(((Tk_FakeWin *) (tkwin))->classUid)
#define Tk_X(tkwin)			(((Tk_FakeWin *) (tkwin))->changes.x)
#define Tk_Y(tkwin)			(((Tk_FakeWin *) (tkwin))->changes.y)
#define Tk_Width(tkwin)			(((Tk_FakeWin *) (tkwin))->changes.width)
#define Tk_Height(tkwin) \
    (((Tk_FakeWin *) (tkwin))->changes.height)
#define Tk_Changes(tkwin)		(&((Tk_FakeWin *) (tkwin))->changes)
#define Tk_Attributes(tkwin)		(&((Tk_FakeWin *) (tkwin))->atts)
#define Tk_IsEmbedded(tkwin) \
    (((Tk_FakeWin *) (tkwin))->flags & TK_EMBEDDED)
#define Tk_IsContainer(tkwin) \
    (((Tk_FakeWin *) (tkwin))->flags & TK_CONTAINER)
#define Tk_IsMapped(tkwin) \
    (((Tk_FakeWin *) (tkwin))->flags & TK_MAPPED)
#define Tk_IsTopLevel(tkwin) \
    (((Tk_FakeWin *) (tkwin))->flags & TK_TOP_LEVEL)
#define Tk_ReqWidth(tkwin)		(((Tk_FakeWin *) (tkwin))->reqWidth)
#define Tk_ReqHeight(tkwin)		(((Tk_FakeWin *) (tkwin))->reqHeight)
#define Tk_InternalBorderWidth(tkwin) \
    (((Tk_FakeWin *) (tkwin))->internalBorderWidth)
#define Tk_Parent(tkwin)		(((Tk_FakeWin *) (tkwin))->parentPtr)
#define Tk_Colormap(tkwin)		(((Tk_FakeWin *) (tkwin))->atts.colormap)

/*
 * The structure below is needed by the macros above so that they can
 * access the fields of a Tk_Window.  The fields not needed by the macros
 * are declared as "dummyX".  The structure has its own type in order to
 * prevent applications from accessing Tk_Window fields except using
 * official macros.  WARNING!! The structure definition must be kept
 * consistent with the TkWindow structure in tkInt.h.  If you change one,
 * then change the other.  See the declaration in tkInt.h for
 * documentation on what the fields are used for internally.
 */

typedef struct Tk_FakeWin {
    Display *display;
    char *dummy1;
    int screenNum;
    Visual *visual;
    int depth;
    Window window;
    char *dummy2;
    char *dummy3;
    Tk_Window parentPtr;
    char *dummy4;
    char *dummy5;
    char *pathName;
    Tk_Uid nameUid;
    Tk_Uid classUid;
    XWindowChanges changes;
    unsigned int dummy6;
    XSetWindowAttributes atts;
    unsigned long dummy7;
    unsigned int flags;
    char *dummy8;
#ifdef TK_USE_INPUT_METHODS
    XIC dummy9;
#endif /* TK_USE_INPUT_METHODS */
    ClientData *dummy10;
    int dummy11;
    int dummy12;
    char *dummy13;
    char *dummy14;
    ClientData dummy15;
    int reqWidth, reqHeight;
    int internalBorderWidth;
    char *dummy16;
    char *dummy17;
    ClientData dummy18;
    char *dummy19;
} Tk_FakeWin;

/*
 * Flag values for TkWindow (and Tk_FakeWin) structures are:
 *
 * TK_MAPPED:			1 means window is currently mapped,
 *				0 means unmapped.
 * TK_TOP_LEVEL:		1 means this is a top-level window (it
 *				was or will be created as a child of
 *				a root window).
 * TK_ALREADY_DEAD:		1 means the window is in the process of
 *				being destroyed already.
 * TK_NEED_CONFIG_NOTIFY:	1 means that the window has been reconfigured
 *				before it was made to exist.  At the time of
 *				making it exist a ConfigureNotify event needs
 *				to be generated.
 * TK_GRAB_FLAG:		Used to manage grabs.  See tkGrab.c for
 *				details.
 * TK_CHECKED_IC:		1 means we've already tried to get an input
 *				context for this window;  if the ic field
 *				is NULL it means that there isn't a context
 *				for the field.
 * TK_DONT_DESTROY_WINDOW:	1 means that Tk_DestroyWindow should not
 *				invoke XDestroyWindow to destroy this widget's
 *				X window.  The flag is set when the window
 *				has already been destroyed elsewhere (e.g.
 *				by another application) or when it will be
 *				destroyed later (e.g. by destroying its
 *				parent).
 * TK_WM_COLORMAP_WINDOW:	1 means that this window has at some time
 *				appeared in the WM_COLORMAP_WINDOWS property
 *				for its toplevel, so we have to remove it
 *				from that property if the window is
 *				deleted and the toplevel isn't.
 * TK_EMBEDDED:			1 means that this window (which must be a
 *				toplevel) is not a free-standing window but
 *				rather is embedded in some other application.
 * TK_CONTAINER:		1 means that this window is a container, and
 *				that some other application (either in
 *				this process or elsewhere) may be
 *				embedding itself inside the window.
 * TK_BOTH_HALVES:		1 means that this window is used for
 *				application embedding (either as
 *				container or embedded application), and
 *				both the containing and embedded halves
 *				are associated with windows in this
 *				particular process.
 * TK_DEFER_MODAL:		1 means that this window has deferred a modal
 *				loop until all of the bindings for the current
 *				event have been invoked.
 * TK_WRAPPER:			1 means that this window is the extra
 *				wrapper window created around a toplevel
 *				to hold the menubar under Unix.  See
 *				tkUnixWm.c for more information.
 * TK_REPARENTED:		1 means that this window has been reparented
 *				so that as far as the window system is
 *				concerned it isn't a child of its Tk
 *				parent.  Initially this is used only for
 *				special Unix menubar windows.
 */


#define TK_MAPPED		1
#define TK_TOP_LEVEL		2
#define TK_ALREADY_DEAD		4
#define TK_NEED_CONFIG_NOTIFY	8
#define TK_GRAB_FLAG		0x10
#define TK_CHECKED_IC		0x20
#define TK_DONT_DESTROY_WINDOW	0x40
#define TK_WM_COLORMAP_WINDOW	0x80
#define TK_EMBEDDED		0x100
#define TK_CONTAINER		0x200
#define TK_BOTH_HALVES		0x400
#define TK_DEFER_MODAL		0x800
#define TK_WRAPPER		0x1000
#define TK_REPARENTED		0x2000

/*
 *--------------------------------------------------------------
 *
 * Procedure prototypes and structures used for defining new canvas
 * items:
 *
 *--------------------------------------------------------------
 */

typedef enum {
    TK_STATE_NULL, TK_STATE_NORMAL, TK_STATE_HIDDEN,
    TK_STATE_DISABLED, TK_STATE_ACTIVE
} Tk_State;

typedef struct Tk_SmoothMethod {
    char *name;
    int (*coordProc) _ANSI_ARGS_((Tk_Canvas canvas,
		double *pointPtr, int numPoints, int numSteps,
		XPoint xPoints[], double dblPoints[]));
    void (*postscriptProc) _ANSI_ARGS_((Tcl_Interp *interp,
		Tk_Canvas canvas, double *coordPtr,
		int numPoints, int numSteps));
} Tk_SmoothMethod;

/*
 * For each item in a canvas widget there exists one record with
 * the following structure.  Each actual item is represented by
 * a record with the following stuff at its beginning, plus additional
 * type-specific stuff after that.
 */

#define TK_TAG_SPACE 3

typedef struct Tk_Item  {
    int id;				/* Unique identifier for this item
					 * (also serves as first tag for
					 * item). */
    struct Tk_Item *nextPtr;		/* Next in display list of all
					 * items in this canvas.  Later items
					 * in list are drawn on top of earlier
					 * ones. */
    Tk_Uid staticTagSpace[TK_TAG_SPACE];/* Built-in space for limited # of
					 * tags. */
    Tk_Uid *tagPtr;			/* Pointer to array of tags.  Usually
					 * points to staticTagSpace, but
					 * may point to malloc-ed space if
					 * there are lots of tags. */
    int tagSpace;			/* Total amount of tag space available
					 * at tagPtr. */
    int numTags;			/* Number of tag slots actually used
					 * at *tagPtr. */
    struct Tk_ItemType *typePtr;	/* Table of procedures that implement
					 * this type of item. */
    int x1, y1, x2, y2;			/* Bounding box for item, in integer
					 * canvas units. Set by item-specific
					 * code and guaranteed to contain every
					 * pixel drawn in item.  Item area
					 * includes x1 and y1 but not x2
					 * and y2. */
    struct Tk_Item *prevPtr;		/* Previous in display list of all
					 * items in this canvas. Later items
					 * in list are drawn just below earlier
					 * ones. */
    Tk_State state;			/* state of item */
    char *updateCmd;			/* for "-updatecommand" option */
    int redraw_flags;			/* some flags used in the canvas */

    /*
     *------------------------------------------------------------------
     * Starting here is additional type-specific stuff;  see the
     * declarations for individual types to see what is part of
     * each type.  The actual space below is determined by the
     * "itemInfoSize" of the type's Tk_ItemType record.
     *------------------------------------------------------------------
     */
} Tk_Item;

/*
 * Flag bits for canvases (redraw_flags):
 *
 * TK_ITEM_STATE_DEPENDANT -	1 means that object needs to be
 *				redrawn if the canvas state changes.
 * TK_ITEM_DONT_REDRAW - 	1 means that the object redraw is already
 *				been prepared, so the general canvas code
 *				doesn't need to do that any more.
 */

#define TK_ITEM_STATE_DEPENDANT		1
#define TK_ITEM_DONT_REDRAW		2

/*
 * Structures used for implementing visitors.
 */

typedef ClientData Tk_VisitorStartProc _ANSI_ARGS_((Tcl_Interp *interp,
		   Tk_Canvas canvas, int argc, char **argv));
typedef int 	Tk_VisitorItemProc _ANSI_ARGS_((Tcl_Interp *interp,
		   Tk_Canvas canvas, ClientData clientData,
		   Tk_Item *itemPtr));
typedef int 	Tk_VisitorEndProc _ANSI_ARGS_((Tcl_Interp *interp,
		   Tk_Canvas canvas, ClientData clientData));

typedef struct Tk_VisitorType {
    char *name;
    int typeSize;			/* should be sizeof(Tk_VisitorType) */
    int flags;				/* If this flag is non-zero then
					 * the item needs to be redrawn. */
    Tk_VisitorStartProc *startProc;
    Tk_VisitorEndProc  *endProc;
    Tk_VisitorItemProc *visitArc;	/* The Arc visitor procedure. */
    Tk_VisitorItemProc *visitBitmap;	/* The Bitmap visitor procedure. */
    Tk_VisitorItemProc *visitImage;	/* The Image visitor procedure. */
    Tk_VisitorItemProc *visitLine;	/* The Line visitor procedure. */
    Tk_VisitorItemProc *visitOval;	/* The Oval visitor procedure. */
    Tk_VisitorItemProc *visitPolygon;	/* The Polygon visitor procedure. */
    Tk_VisitorItemProc *visitRectangle;	/* The Rectangle visitor procedure. */
    Tk_VisitorItemProc *visitText;	/* The Text visitor procedure. */
    Tk_VisitorItemProc *visitWindow;	/* The Window visitor procedure. */
    Tk_VisitorItemProc *visitSticker;	/* The Sticker visitor procedure.
					 * from tkSticker*/
    Tk_VisitorItemProc *visitViewport;	/* The Viewport visitor procedure.
					 * from TkSM */
    Tk_VisitorItemProc *visitBarchart;	/* The Barchart visitor procedure.
					 * from Scotty */
    Tk_VisitorItemProc *visitStripchart;/* The Stripchart visitor procedure.
					 * from Scotty */

    /*
     *------------------------------------------------------------------
     * Starting here is additional item-extension stuff;
     * you can add extra item types after this plus other information.
     *------------------------------------------------------------------
     */
} Tk_VisitorType;

/*
 * Records of the following type are used to describe a type of
 * item (e.g.  lines, circles, etc.) that can form part of a
 * canvas widget.
 */

typedef int	Tk_ItemCreateProc _ANSI_ARGS_((Tcl_Interp *interp,
		    Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
		    char **argv));
typedef int	Tk_ItemConfigureProc _ANSI_ARGS_((Tcl_Interp *interp,
		    Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
		    char **argv, int flags));
typedef int	Tk_ItemCoordProc _ANSI_ARGS_((Tcl_Interp *interp,
		    Tk_Canvas canvas, Tk_Item *itemPtr, int argc,
		    char **argv));
typedef void	Tk_ItemDeleteProc _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *itemPtr, Display *display));
typedef void	Tk_ItemDisplayProc _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *itemPtr, Display *display, Drawable dst,
		    int x, int y, int width, int height));
typedef double	Tk_ItemPointProc _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *itemPtr, double *pointPtr));
typedef int	Tk_ItemAreaProc _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *itemPtr, double *rectPtr));
typedef int	Tk_ItemPostscriptProc _ANSI_ARGS_((Tcl_Interp *interp,
		    Tk_Canvas canvas, Tk_Item *itemPtr, int prepass));
typedef void	Tk_ItemScaleProc _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *itemPtr, double originX, double originY,
		    double scaleX, double scaleY));
typedef void	Tk_ItemTranslateProc _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *itemPtr, double deltaX, double deltaY));
typedef int	Tk_ItemIndexProc _ANSI_ARGS_((Tcl_Interp *interp,
		    Tk_Canvas canvas, Tk_Item *itemPtr, char *indexString,
		    int *indexPtr));
typedef void	Tk_ItemCursorProc _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *itemPtr, int index));
typedef int	Tk_ItemSelectionProc _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *itemPtr, int offset, char *buffer,
		    int maxBytes));
typedef void	Tk_ItemInsertProc _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *itemPtr, int beforeThis, char *string));
typedef void	Tk_ItemDCharsProc _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *itemPtr, int first, int last));
typedef void	Tk_ItemBboxProc _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *itemPtr));
typedef int	Tk_ItemGetCoordProc _ANSI_ARGS_((Tk_Item *itemPtr,
		    double **coordsPtr, Tcl_FreeProc *freeProc));
typedef int	Tk_ItemSetCoordProc _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *itemPtr, int num, double *coords));

/*
 * Some flag values that can be used in the "flags" field in Tk_ItemType.
 */

#define TK_ITEM_ALWAYS_REDRAW 1
#define TK_ITEM_VISITOR_SUPPORT 2

/*
 * Structures used for implementing visitors.
 */

typedef struct Tk_ItemType {
    char *name;				/* The name of this type of item, such
					 * as "line". */
    int itemSize;			/* Total amount of space needed for
					 * item's record. */
    Tk_ItemCreateProc *createProc;	/* Procedure to create a new item of
					 * this type. */
    Tk_ConfigSpec *configSpecs;		/* Pointer to array of configuration
					 * specs for this type.  Used for
					 * returning configuration info. */
    Tk_ItemConfigureProc *configProc;	/* Procedure to call to change
					 * configuration options. */
    Tk_ItemCoordProc *coordProc;	/* Procedure to call to get and set
					 * the item's coordinates. */
    Tk_ItemDeleteProc *deleteProc;	/* Procedure to delete existing item of
					 * this type. */
    Tk_ItemDisplayProc *displayProc;	/* Procedure to display items of
					 * this type. */
    int flags;				/* TK_ITEM_TYPE_ALWAYS_REDRAW means displayProc
					 * should be called even when the item has
					 * been moved off-screen. */
    Tk_ItemPointProc *pointProc;	/* Computes distance from item to
					 * a given point. */
    Tk_ItemAreaProc *areaProc;		/* Computes whether item is inside,
					 * outside, or overlapping an area. */
    Tk_ItemPostscriptProc *postscriptProc;
					/* Procedure to write a Postscript
					 * description for items of this
					 * type. */
    Tk_ItemScaleProc *scaleProc;	/* Procedure to rescale items of
					 * this type. */
    Tk_ItemTranslateProc *translateProc;/* Procedure to translate items of
					 * this type. */
    Tk_ItemIndexProc *indexProc;	/* Procedure to determine index of
					 * indicated character.  NULL if
					 * item doesn't support indexing. */
    Tk_ItemCursorProc *icursorProc;	/* Procedure to set insert cursor pos.
					 * to just before a given position. */
    Tk_ItemSelectionProc *selectionProc;/* Procedure to return selection (in
					 * STRING format) when it is in this
					 * item. */
    Tk_ItemInsertProc *insertProc;	/* Procedure to insert something into
					 * an item. */
    Tk_ItemDCharsProc *dCharsProc;	/* Procedure to delete characters
					 * from an item. */
    struct Tk_ItemType *nextPtr;	/* Used to link types together into
					 * a list. */
    Tk_ItemBboxProc *bboxProc;		/* Procedure to calculate the bounding
					 * box for an item. */
    int acceptProc;			/* Offset of visitors function  in
					 * tkVisitorType struct for this item. */
    Tk_ItemGetCoordProc *getCoordProc;	/* Procedure to get the coordinates
					 * for an item. */
    Tk_ItemSetCoordProc *setCoordProc;	/* Procedure to set the coordinates
					 * for an item. */
} Tk_ItemType;

/*
 * The following structure provides information about the selection and
 * the insertion cursor.  It is needed by only a few items, such as
 * those that display text.  It is shared by the generic canvas code
 * and the item-specific code, but most of the fields should be written
 * only by the canvas generic code.
 */

typedef struct Tk_CanvasTextInfo {
    Tk_3DBorder selBorder;	/* Border and background for selected
				 * characters.  Read-only to items.*/
    int selBorderWidth;		/* Width of border around selection. 
				 * Read-only to items. */
    XColor *selFgColorPtr;	/* Foreground color for selected text.
				 * Read-only to items. */
    Tk_Item *selItemPtr;	/* Pointer to selected item.  NULL means
				 * selection isn't in this canvas.
				 * Writable by items. */
    int selectFirst;		/* Index of first selected character. 
				 * Writable by items. */
    int selectLast;		/* Index of last selected character. 
				 * Writable by items. */
    Tk_Item *anchorItemPtr;	/* Item corresponding to "selectAnchor":
				 * not necessarily selItemPtr.   Read-only
				 * to items. */
    int selectAnchor;		/* Fixed end of selection (i.e. "select to"
				 * operation will use this as one end of the
				 * selection).  Writable by items. */
    Tk_3DBorder insertBorder;	/* Used to draw vertical bar for insertion
				 * cursor.  Read-only to items. */
    int insertWidth;		/* Total width of insertion cursor.  Read-only
				 * to items. */
    int insertBorderWidth;	/* Width of 3-D border around insert cursor.
				 * Read-only to items. */
    Tk_Item *focusItemPtr;	/* Item that currently has the input focus,
				 * or NULL if no such item.  Read-only to
				 * items.  */
    int gotFocus;		/* Non-zero means that the canvas widget has
				 * the input focus.  Read-only to items.*/
    int cursorOn;		/* Non-zero means that an insertion cursor
				 * should be displayed in focusItemPtr.
				 * Read-only to items.*/
} Tk_CanvasTextInfo;

/*
 * Structures used for Dashing and Outline.
 */

typedef struct Tk_Dash {
    int number;
    union {
	char *pt;
	char array[sizeof(char *)];
    } pattern;
} Tk_Dash;

typedef struct Tk_TSOffset {
    int flags;			/* flags; see below for possible values */
    int xoffset;		/* x offset */
    int yoffset;		/* y offset */
} Tk_TSOffset;

/*
 * Bit fields in Tk_Offset->flags:
 */

#define TK_OFFSET_INDEX		1
#define TK_OFFSET_RELATIVE	2
#define TK_OFFSET_LEFT		4
#define TK_OFFSET_CENTER	8
#define TK_OFFSET_RIGHT		16
#define TK_OFFSET_TOP		32
#define TK_OFFSET_MIDDLE	64
#define TK_OFFSET_BOTTOM	128

typedef struct Tk_Tile_ *Tk_Tile;	/* Opaque type for tiles */

typedef struct Tk_Outline {
    GC gc;			/* Graphics context. */
    double width;		/* Width of outline. */
    double activeWidth;		/* Width of outline. */
    double disabledWidth;	/* Width of outline. */
    int offset;			/* Dash offset */
    Tk_Dash dash;		/* Dash pattern */
    Tk_Dash activeDash;		/* Dash pattern if state is active*/
    Tk_Dash disabledDash;	/* Dash pattern if state is disabled*/
    Tk_Tile tile;		/* Tile pattern */
    Tk_Tile activeTile;		/* Tile pattern if state is active*/
    Tk_Tile disabledTile;	/* Tile pattern if state is disabled*/
    Tk_TSOffset tsoffset;	/* tile/stipple offset for outline*/
    XColor *color;		/* Outline color. */
    XColor *activeColor;	/* Outline color if state is active. */
    XColor *disabledColor;	/* Outline color if state is disabled. */
    Pixmap stipple;		/* Outline Stipple pattern. */
    Pixmap activeStipple;	/* Outline Stipple pattern if state is active. */
    Pixmap disabledStipple;	/* Outline Stipple pattern if state is disabled. */
} Tk_Outline;

/*
 * Some functions handy for Dashing and Outlines (in tkCanvUtil.c).
 */

EXTERN int	Tk_GetDash _ANSI_ARGS_((Tcl_Interp *interp,
		    CONST char *value, Tk_Dash *dash));
EXTERN void	Tk_CreateOutline _ANSI_ARGS_((Tk_Outline *outline));
EXTERN void	Tk_DeleteOutline _ANSI_ARGS_((Display *display,
		    Tk_Outline *outline));
EXTERN int	Tk_ConfigOutlineGC _ANSI_ARGS_((XGCValues *gcValues,
		    Tk_Canvas canvas ,Tk_Item *item,
		    Tk_Outline *outline));
EXTERN int	Tk_ChangeOutlineGC _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *item, Tk_Outline *outline));
EXTERN int	Tk_ResetOutlineGC _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *item, Tk_Outline *outline));
EXTERN int	Tk_CanvasPsOutline _ANSI_ARGS_((Tk_Canvas canvas,
		    Tk_Item *item, Tk_Outline *outline));

/*
 * Some functions handy for Tiling and Stipple/Tile offset.
 */

typedef void	(Tk_TileChangedProc) _ANSI_ARGS_((ClientData clientData,
		    Tk_Tile tile, Tk_Item *itemPtr));
EXTERN Tk_Tile	Tk_GetTile _ANSI_ARGS_((Tcl_Interp *interp, Tk_Window tkwin,
		    CONST char *imageName));
EXTERN void	Tk_FreeTile _ANSI_ARGS_((Tk_Tile tile));
EXTERN char *	Tk_NameOfTile _ANSI_ARGS_((Tk_Tile tile));
EXTERN void	Tk_SetTileChangedProc _ANSI_ARGS_((Tk_Tile tile,
		    Tk_TileChangedProc * changeProc, ClientData clientData,
		    Tk_Item *itemPtr));
EXTERN Pixmap	Tk_PixmapOfTile _ANSI_ARGS_((Tk_Tile tile));
EXTERN int	Tk_PixelParseProc _ANSI_ARGS_((
		    ClientData clientData, Tcl_Interp *interp,
		    Tk_Window tkwin, CONST char *value, char *widgRec,
		    int offset));
EXTERN char *	Tk_PixelPrintProc _ANSI_ARGS_((
		    ClientData clientData, Tk_Window tkwin,
		    char *widgRec, int offset,
		    Tcl_FreeProc **freeProcPtr));
EXTERN void	Tk_SizeOfTile _ANSI_ARGS_((Tk_Tile tile, int *widthPtr,
		    int *heightPtr));
EXTERN void	Tk_SetTileOrigin _ANSI_ARGS_((Tk_Window tkwin, GC gc, int x,
		    int y));
EXTERN int	Tk_TileParseProc _ANSI_ARGS_((
		    ClientData clientData, Tcl_Interp *interp,
		    Tk_Window tkwin, CONST char *value, char *widgRec,
		    int offset));
EXTERN char *	Tk_TilePrintProc _ANSI_ARGS_((
		    ClientData clientData, Tk_Window tkwin,
		    char *widgRec, int offset,
		    Tcl_FreeProc **freeProcPtr));
EXTERN int	Tk_OffsetParseProc _ANSI_ARGS_((
		    ClientData clientData, Tcl_Interp *interp,
		    Tk_Window tkwin, CONST char *value, char *widgRec,
		    int offset));
EXTERN char *	Tk_OffsetPrintProc _ANSI_ARGS_((
		    ClientData clientData, Tk_Window tkwin,
		    char *widgRec, int offset,
		    Tcl_FreeProc **freeProcPtr));
EXTERN void	Tk_SetTileCanvasItem _ANSI_ARGS_((Tk_Tile tile,
			    Tk_Canvas canvas, Tk_Item *itemPtr));


/*
 *--------------------------------------------------------------
 *
 * Procedure prototypes and structures used for managing images:
 *
 *--------------------------------------------------------------
 */

typedef struct Tk_ImageType Tk_ImageType;
typedef int (Tk_ImageCreateProc) _ANSI_ARGS_((Tcl_Interp *interp,
	char *name, int objc, Tcl_Obj *CONST objv[], Tk_ImageType *typePtr,
	Tk_ImageMaster master, ClientData *masterDataPtr));
typedef ClientData (Tk_ImageGetProc) _ANSI_ARGS_((Tk_Window tkwin,
	ClientData masterData));
typedef void (Tk_ImageDisplayProc) _ANSI_ARGS_((ClientData instanceData,
	Display *display, Drawable drawable, int imageX, int imageY,
	int width, int height, int drawableX, int drawableY));
typedef void (Tk_ImageFreeProc) _ANSI_ARGS_((ClientData instanceData,
	Display *display));
typedef void (Tk_ImageDeleteProc) _ANSI_ARGS_((ClientData masterData));
typedef void (Tk_ImageChangedProc) _ANSI_ARGS_((ClientData clientData,
	int x, int y, int width, int height, int imageWidth,
	int imageHeight));
typedef int (Tk_ImagePostscriptProc) _ANSI_ARGS_((ClientData clientData,
	Tcl_Interp *interp, Tk_Window tkwin, Tk_PostscriptInfo psinfo,
	int x, int y, int width, int height, int prepass));

/*
 * The following structure represents a particular type of image
 * (bitmap, xpm image, etc.).  It provides information common to
 * all images of that type, such as the type name and a collection
 * of procedures in the image manager that respond to various
 * events.  Each image manager is represented by one of these
 * structures.
 */

struct Tk_ImageType {
    char *name;			/* Name of image type. */
    Tk_ImageCreateProc *createProc;
				/* Procedure to call to create a new image
				 * of this type. */
    Tk_ImageGetProc *getProc;	/* Procedure to call the first time
				 * Tk_GetImage is called in a new way
				 * (new visual or screen). */
    Tk_ImageDisplayProc *displayProc;
				/* Call to draw image, in response to
				 * Tk_RedrawImage calls. */
    Tk_ImageFreeProc *freeProc;	/* Procedure to call whenever Tk_FreeImage
				 * is called to release an instance of an
				 * image. */
    Tk_ImageDeleteProc *deleteProc;
				/* Procedure to call to delete image.  It
				 * will not be called until after freeProc
				 * has been called for each instance of the
				 * image. */
    Tk_ImagePostscriptProc *postscriptProc;
				/* Procedure to call to create postscript
				 * output for this image type. */
    struct Tk_ImageType *nextPtr;
				/* Next in list of all image types currently
				 * known.  Filled in by Tk, not by image
				 * manager. */
    char *reserved;		/* reserved for future expansion */
};

/*
 *--------------------------------------------------------------
 *
 * Additional definitions used to manage images of type "photo".
 *
 *--------------------------------------------------------------
 */

/*
 * The following type is used to identify a particular photo image
 * to be manipulated:
 */

typedef void *Tk_PhotoHandle;

/*
 * The following structure describes a block of pixels in memory:
 */

typedef struct Tk_PhotoImageBlock {
    unsigned char *pixelPtr;	/* Pointer to the first pixel. */
    int		width;		/* Width of block, in pixels. */
    int		height;		/* Height of block, in pixels. */
    int		pitch;		/* Address difference between corresponding
				 * pixels in successive lines. */
    int		pixelSize;	/* Address difference between successive
				 * pixels in the same line. */
    int		offset[3];	/* Address differences between the red, green
				 * and blue components of the pixel and the
				 * pixel as a whole. */
    int		reserved;	/* Reserved for extensions (dash patch) */
} Tk_PhotoImageBlock;

/*
 * Procedure prototypes and structures used in reading and
 * writing photo images:
 */

typedef struct Tk_PhotoImageFormat Tk_PhotoImageFormat;
typedef int (Tk_ImageFileMatchProc) _ANSI_ARGS_((Tcl_Channel chan,
	char *fileName, Tcl_Obj *format, int *widthPtr,
	int *heightPtr, Tcl_Interp *interp));
typedef int (Tk_ImageStringMatchProc) _ANSI_ARGS_((Tcl_Obj *dataObj,
	Tcl_Obj *format, int *widthPtr, int *heightPtr,
	Tcl_Interp *interp));
typedef int (Tk_ImageFileReadProc) _ANSI_ARGS_((Tcl_Interp *interp,
	Tcl_Channel chan, char *fileName, Tcl_Obj *format,
	Tk_PhotoHandle imageHandle, int destX, int destY,
	int width, int height, int srcX, int srcY));
typedef int (Tk_ImageStringReadProc) _ANSI_ARGS_((Tcl_Interp *interp,
	Tcl_Obj *dataObj, Tcl_Obj *format, Tk_PhotoHandle imageHandle,
	int destX, int destY, int width, int height, int srcX, int srcY));
typedef int (Tk_ImageFileWriteProc) _ANSI_ARGS_((Tcl_Interp *interp,
	char *fileName, Tcl_Obj *format, Tk_PhotoImageBlock *blockPtr));
typedef int (Tk_ImageStringWriteProc) _ANSI_ARGS_((Tcl_Interp *interp,
	Tcl_DString *dataPtr, Tcl_Obj *format,
	Tk_PhotoImageBlock *blockPtr));

/*
 * The following structure represents a particular file format for
 * storing images (e.g., PPM, GIF, JPEG, etc.).  It provides information
 * to allow image files of that format to be recognized and read into
 * a photo image.
 */

struct Tk_PhotoImageFormat {
    char *name;			/* Name of image file format */
    Tk_ImageFileMatchProc *fileMatchProc;
				/* Procedure to call to determine whether
				 * an image file matches this format. */
    Tk_ImageStringMatchProc *stringMatchProc;
				/* Procedure to call to determine whether
				 * the data in a string matches this format. */
    Tk_ImageFileReadProc *fileReadProc;
				/* Procedure to call to read data from
				 * an image file into a photo image. */
    Tk_ImageStringReadProc *stringReadProc;
				/* Procedure to call to read data from
				 * a string into a photo image. */
    Tk_ImageFileWriteProc *fileWriteProc;
				/* Procedure to call to write data from
				 * a photo image to a file. */
    Tk_ImageStringWriteProc *stringWriteProc;
				/* Procedure to call to obtain a string
				 * representation of the data in a photo
				 * image.*/
    struct Tk_PhotoImageFormat *nextPtr;
				/* Next in list of all photo image formats
				 * currently known.  Filled in by Tk, not
				 * by image format handler. */
};

/*
 *--------------------------------------------------------------
 *
 * The definitions below provide backward compatibility for
 * functions and types related to event handling that used to
 * be in Tk but have moved to Tcl.
 *
 *--------------------------------------------------------------
 */

#define TK_READABLE		TCL_READABLE
#define TK_WRITABLE		TCL_WRITABLE
#define TK_EXCEPTION		TCL_EXCEPTION

#define TK_DONT_WAIT		TCL_DONT_WAIT
#define TK_X_EVENTS		TCL_WINDOW_EVENTS
#define TK_WINDOW_EVENTS	TCL_WINDOW_EVENTS
#define TK_FILE_EVENTS		TCL_FILE_EVENTS
#define TK_TIMER_EVENTS		TCL_TIMER_EVENTS
#define TK_IDLE_EVENTS		TCL_IDLE_EVENTS
#define TK_ALL_EVENTS		TCL_ALL_EVENTS

#define Tk_IdleProc		Tcl_IdleProc
#define Tk_FileProc		Tcl_FileProc
#define Tk_TimerProc		Tcl_TimerProc
#define Tk_TimerToken		Tcl_TimerToken

#define Tk_BackgroundError	Tcl_BackgroundError
#define Tk_CancelIdleCall	Tcl_CancelIdleCall
#define Tk_CreateFileHandler	Tcl_CreateFileHandler
#define Tk_CreateTimerHandler	Tcl_CreateTimerHandler
#define Tk_DeleteFileHandler	Tcl_DeleteFileHandler
#define Tk_DeleteTimerHandler	Tcl_DeleteTimerHandler
#define Tk_DoOneEvent		Tcl_DoOneEvent
#define Tk_DoWhenIdle		Tcl_DoWhenIdle
#define Tk_Sleep		Tcl_Sleep

#define Tk_EventuallyFree	Tcl_EventuallyFree
#define Tk_FreeProc		Tcl_FreeProc
#define Tk_Preserve		Tcl_Preserve
#define Tk_Release		Tcl_Release
#define Tk_FileeventCmd		Tcl_FileEventCmd

/*
 *--------------------------------------------------------------
 *
 * Additional procedure types defined by Tk.
 *
 *--------------------------------------------------------------
 */

typedef int (Tk_ErrorProc) _ANSI_ARGS_((ClientData clientData,
	XErrorEvent *errEventPtr));
typedef void (Tk_EventProc) _ANSI_ARGS_((ClientData clientData,
	XEvent *eventPtr));
typedef int (Tk_GenericProc) _ANSI_ARGS_((ClientData clientData,
	XEvent *eventPtr));
typedef int (Tk_GetSelProc) _ANSI_ARGS_((ClientData clientData,
	Tcl_Interp *interp, char *portion));
typedef void (Tk_LostSelProc) _ANSI_ARGS_((ClientData clientData));
typedef Tk_RestrictAction (Tk_RestrictProc) _ANSI_ARGS_((
	ClientData clientData, XEvent *eventPtr));
typedef int (Tk_SelectionProc) _ANSI_ARGS_((ClientData clientData,
	int offset, char *buffer, int maxBytes));

/*
 *--------------------------------------------------------------
 *
 * Exported procedures and variables.
 *
 *--------------------------------------------------------------
 */

EXTERN XColor *		Tk_3DBorderColor _ANSI_ARGS_((Tk_3DBorder border));
EXTERN GC		Tk_3DBorderGC _ANSI_ARGS_((Tk_Window tkwin,
			    Tk_3DBorder border, int which));
EXTERN void		Tk_3DHorizontalBevel _ANSI_ARGS_((Tk_Window tkwin,
			    Drawable drawable, Tk_3DBorder border, int x,
			    int y, int width, int height, int leftIn,
			    int rightIn, int topBevel, int relief));
EXTERN void		Tk_3DVerticalBevel _ANSI_ARGS_((Tk_Window tkwin,
			    Drawable drawable, Tk_3DBorder border, int x,
			    int y, int width, int height, int leftBevel,
			    int relief));
EXTERN void		Tk_AddOption _ANSI_ARGS_((Tk_Window tkwin, char *name,
			    char *value, int priority));
EXTERN void		Tk_BindEvent _ANSI_ARGS_((Tk_BindingTable bindingTable,
			    XEvent *eventPtr, Tk_Window tkwin, int numObjects,
			    ClientData *objectPtr));
EXTERN int		Tk_CanvasDashParseProc _ANSI_ARGS_((
			    ClientData clientData, Tcl_Interp *interp,
			    Tk_Window tkwin, CONST char *value, char *widgRec,
			    int offset));
EXTERN char *		Tk_CanvasDashPrintProc _ANSI_ARGS_((
			    ClientData clientData, Tk_Window tkwin,
			    char *widgRec, int offset,
			    Tcl_FreeProc **freeProcPtr));
EXTERN void		Tk_CanvasDrawableCoords _ANSI_ARGS_((Tk_Canvas canvas,
			    double x, double y, short *drawableXPtr,
			    short *drawableYPtr));
EXTERN void		Tk_CanvasEventuallyRedraw _ANSI_ARGS_((
			    Tk_Canvas canvas, int x1, int y1, int x2,
			    int y2));
EXTERN int		Tk_CanvasGetCoord _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Canvas canvas, char *string,
			    double *doublePtr));
EXTERN int		Tk_CanvasGetCoordFromObj _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Canvas canvas, Tcl_Obj *obj,
			    double *doublePtr));
EXTERN Tk_CanvasTextInfo *Tk_CanvasGetTextInfo _ANSI_ARGS_((Tk_Canvas canvas));
EXTERN int		Tk_CanvasPsBitmap _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Canvas canvas, Pixmap bitmap, int x, int y,
			    int width, int height));
EXTERN int		Tk_CanvasPsColor _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Canvas canvas, XColor *colorPtr));
EXTERN int		Tk_CanvasPsFont _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Canvas canvas, Tk_Font font));
EXTERN void		Tk_CanvasPsPath _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Canvas canvas, double *coordPtr, int numPoints));
EXTERN int		Tk_CanvasPsStipple _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Canvas canvas, Pixmap bitmap));
EXTERN double		Tk_CanvasPsY _ANSI_ARGS_((Tk_Canvas canvas, double y));
EXTERN void		Tk_CanvasSetOffset _ANSI_ARGS_((
			    Tk_Canvas canvas, GC gc, Tk_TSOffset *offset));
EXTERN void		Tk_CanvasSetStippleOrigin _ANSI_ARGS_((
			    Tk_Canvas canvas, GC gc));
EXTERN int		Tk_CanvasTagsParseProc _ANSI_ARGS_((
			    ClientData clientData, Tcl_Interp *interp,
			    Tk_Window tkwin, char *value, char *widgRec,
			    int offset));
EXTERN char *		Tk_CanvasTagsPrintProc _ANSI_ARGS_((
			    ClientData clientData, Tk_Window tkwin,
			    char *widgRec, int offset,
			    Tcl_FreeProc **freeProcPtr));
EXTERN Tk_Window	Tk_CanvasTkwin _ANSI_ARGS_((Tk_Canvas canvas));
EXTERN void		Tk_CanvasWindowCoords _ANSI_ARGS_((Tk_Canvas canvas,
			    double x, double y, short *screenXPtr,
			    short *screenYPtr));
EXTERN void		Tk_ChangeWindowAttributes _ANSI_ARGS_((Tk_Window tkwin,
			    unsigned long valueMask,
			    XSetWindowAttributes *attsPtr));
EXTERN int		Tk_CharBbox _ANSI_ARGS_((Tk_TextLayout layout,
			    int index, int *xPtr, int *yPtr, int *widthPtr,
			    int *heightPtr));
EXTERN void		Tk_ClearSelection _ANSI_ARGS_((Tk_Window tkwin,
			    Atom selection));
EXTERN int		Tk_ClipboardAppend _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, Atom target, Atom format,
			    char* buffer));
EXTERN int		Tk_ClipboardClear _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin));
EXTERN int		Tk_ConfigureInfo _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, Tk_ConfigSpec *specs,
			    char *widgRec, char *argvName, int flags));
EXTERN int		Tk_ConfigureValue _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, Tk_ConfigSpec *specs,
			    char *widgRec, char *argvName, int flags));
EXTERN int		Tk_ConfigureWidget _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, Tk_ConfigSpec *specs,
			    int argc, char **argv, char *widgRec,
			    int flags));
EXTERN void		Tk_ConfigureWindow _ANSI_ARGS_((Tk_Window tkwin,
			    unsigned int valueMask, XWindowChanges *valuePtr));
EXTERN Tk_TextLayout	Tk_ComputeTextLayout _ANSI_ARGS_((Tk_Font font,
			    CONST char *string, int numChars, int wrapLength,
			    Tk_Justify justify, int flags, int *widthPtr,
			    int *heightPtr));
EXTERN Tk_Window	Tk_CoordsToWindow _ANSI_ARGS_((int rootX, int rootY,
			    Tk_Window tkwin));
EXTERN unsigned long	Tk_CreateBinding _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_BindingTable bindingTable, ClientData object,
			    char *eventString, char *command, int append));
EXTERN Tk_BindingTable	Tk_CreateBindingTable _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN Tk_ErrorHandler	Tk_CreateErrorHandler _ANSI_ARGS_((Display *display,
			    int errNum, int request, int minorCode,
			    Tk_ErrorProc *errorProc, ClientData clientData));
EXTERN void		Tk_CreateEventHandler _ANSI_ARGS_((Tk_Window token,
			    unsigned long mask, Tk_EventProc *proc,
			    ClientData clientData));
EXTERN void		Tk_CreatePhotoOption _ANSI_ARGS_((Tcl_Interp *interp,
			    CONST char *name, Tcl_CmdProc *proc));
EXTERN void		Tk_CreateGenericHandler _ANSI_ARGS_((
			    Tk_GenericProc *proc, ClientData clientData));
EXTERN void		Tk_CreateImageType _ANSI_ARGS_((
			    Tk_ImageType *typePtr));
EXTERN void		Tk_CreateItemType _ANSI_ARGS_((Tk_ItemType *typePtr));
EXTERN void		Tk_CreatePhotoImageFormat _ANSI_ARGS_((
			    Tk_PhotoImageFormat *formatPtr));
EXTERN void		Tk_CreateSelHandler _ANSI_ARGS_((Tk_Window tkwin,
			    Atom selection, Atom target,
			    Tk_SelectionProc *proc, ClientData clientData,
			    Atom format));
EXTERN void		Tk_CreateSmoothMethod _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_SmoothMethod *method));
EXTERN void		Tk_CreateCanvasVisitor _ANSI_ARGS_((Tcl_Interp *interp,
			    CONST Tk_VisitorType *typePtr));
EXTERN Tk_Window	Tk_CreateWindow _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window parent, char *name, char *screenName));
EXTERN Tk_Window	Tk_CreateWindowFromPath _ANSI_ARGS_((
			    Tcl_Interp *interp, Tk_Window tkwin,
			    char *pathName, char *screenName));
EXTERN int		Tk_DefineBitmap _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Uid name, char *source, int width,
			    int height));
EXTERN void		Tk_DefineCursor _ANSI_ARGS_((Tk_Window window,
			    Tk_Cursor cursor));
EXTERN void		Tk_DeleteAllBindings _ANSI_ARGS_((
			    Tk_BindingTable bindingTable, ClientData object));
EXTERN int		Tk_DeleteBinding _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_BindingTable bindingTable, ClientData object,
			    char *eventString));
EXTERN void		Tk_DeleteBindingTable _ANSI_ARGS_((
			    Tk_BindingTable bindingTable));
EXTERN void		Tk_DeleteErrorHandler _ANSI_ARGS_((
			    Tk_ErrorHandler handler));
EXTERN void		Tk_DeleteEventHandler _ANSI_ARGS_((Tk_Window token,
			    unsigned long mask, Tk_EventProc *proc,
			    ClientData clientData));
EXTERN void		Tk_DeleteGenericHandler _ANSI_ARGS_((
			    Tk_GenericProc *proc, ClientData clientData));
EXTERN void		Tk_DeleteImage _ANSI_ARGS_((Tcl_Interp *interp,
			    char *name));
EXTERN void		Tk_DeleteSelHandler _ANSI_ARGS_((Tk_Window tkwin,
			    Atom selection, Atom target));
EXTERN void             Tk_DestroyWindow _ANSI_ARGS_((Tk_Window tkwin));
EXTERN char *		Tk_DisplayName _ANSI_ARGS_((Tk_Window tkwin));
EXTERN void		Tk_DitherPhoto _ANSI_ARGS_((Tk_PhotoHandle handle,
			    int x, int y, int width, int height));
EXTERN int		Tk_DistanceToTextLayout _ANSI_ARGS_((
			    Tk_TextLayout layout, int x, int y));
EXTERN void		Tk_Draw3DPolygon _ANSI_ARGS_((Tk_Window tkwin,
			    Drawable drawable, Tk_3DBorder border,
			    XPoint *pointPtr, int numPoints, int borderWidth,
			    int leftRelief));
EXTERN void		Tk_Draw3DRectangle _ANSI_ARGS_((Tk_Window tkwin,
			    Drawable drawable, Tk_3DBorder border, int x,
			    int y, int width, int height, int borderWidth,
			    int relief));
EXTERN void		Tk_DrawChars _ANSI_ARGS_((Display *display,
			    Drawable drawable, GC gc, Tk_Font tkfont,
			    CONST char *source, int numChars, int x,
			    int y));
EXTERN void		Tk_DrawFocusHighlight _ANSI_ARGS_((Tk_Window tkwin,
			    GC gc, int width, Drawable drawable));
EXTERN void		Tk_DrawTextLayout _ANSI_ARGS_((Display *display,
			    Drawable drawable, GC gc, Tk_TextLayout layout,
			    int x, int y, int firstChar, int lastChar));
EXTERN void		Tk_Fill3DPolygon _ANSI_ARGS_((Tk_Window tkwin,
			    Drawable drawable, Tk_3DBorder border,
			    XPoint *pointPtr, int numPoints, int borderWidth,
			    int leftRelief));
EXTERN void		Tk_Fill3DRectangle _ANSI_ARGS_((Tk_Window tkwin,
			    Drawable drawable, Tk_3DBorder border, int x,
			    int y, int width, int height, int borderWidth,
			    int relief));
EXTERN Tk_PhotoHandle	Tk_FindPhoto _ANSI_ARGS_((Tcl_Interp *interp,
			    char *imageName));
EXTERN Font		Tk_FontId _ANSI_ARGS_((Tk_Font font));
EXTERN void		Tk_Free3DBorder _ANSI_ARGS_((Tk_3DBorder border));
EXTERN void		Tk_FreeBitmap _ANSI_ARGS_((Display *display,
			    Pixmap bitmap));
EXTERN void		Tk_FreeColor _ANSI_ARGS_((XColor *colorPtr));
EXTERN void		Tk_FreeColormap _ANSI_ARGS_((Display *display,
			    Colormap colormap));
EXTERN void		Tk_FreeCursor _ANSI_ARGS_((Display *display,
			    Tk_Cursor cursor));
EXTERN void		Tk_FreeFont _ANSI_ARGS_((Tk_Font));
EXTERN void		Tk_FreeGC _ANSI_ARGS_((Display *display, GC gc));
EXTERN void		Tk_FreeImage _ANSI_ARGS_((Tk_Image image));
EXTERN void		Tk_FreeOptions _ANSI_ARGS_((Tk_ConfigSpec *specs,
			    char *widgRec, Display *display, int needFlags));
EXTERN void		Tk_FreePixmap _ANSI_ARGS_((Display *display,
			    Pixmap pixmap));
EXTERN void		Tk_FreeTextLayout _ANSI_ARGS_((
			    Tk_TextLayout textLayout));
EXTERN void		Tk_FreeXId _ANSI_ARGS_((Display *display, XID xid));
EXTERN GC		Tk_GCForColor _ANSI_ARGS_((XColor *colorPtr,
			    Drawable drawable));
EXTERN void		Tk_GeometryRequest _ANSI_ARGS_((Tk_Window tkwin,
			    int reqWidth,  int reqHeight));
EXTERN Tk_3DBorder	Tk_Get3DBorder _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, Tk_Uid colorName));
EXTERN void		Tk_GetAllBindings _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_BindingTable bindingTable, ClientData object));
EXTERN int		Tk_GetAnchor _ANSI_ARGS_((Tcl_Interp *interp,
			    char *string, Tk_Anchor *anchorPtr));
EXTERN char *		Tk_GetAtomName _ANSI_ARGS_((Tk_Window tkwin,
			    Atom atom));
EXTERN char *		Tk_GetBinding _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_BindingTable bindingTable, ClientData object,
			    char *eventString));
EXTERN Pixmap		Tk_GetBitmap _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, Tk_Uid string));
EXTERN Pixmap		Tk_GetBitmapFromData _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, char *source,
			    int width, int height));
EXTERN Tk_VisitorType *	Tk_GetCanvasVisitor _ANSI_ARGS_((Tcl_Interp *interp,
			    CONST char *name));
EXTERN int		Tk_GetCapStyle _ANSI_ARGS_((Tcl_Interp *interp,
			    char *string, int *capPtr));
EXTERN XColor *		Tk_GetColor _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, Tk_Uid name));
EXTERN XColor *		Tk_GetColorByValue _ANSI_ARGS_((Tk_Window tkwin,
			    XColor *colorPtr));
EXTERN Colormap		Tk_GetColormap _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, char *string));
EXTERN Tk_Cursor	Tk_GetCursor _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, Tk_Uid string));
EXTERN Tk_Cursor	Tk_GetCursorFromData _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, char *source, char *mask,
			    int width, int height, int xHot, int yHot,
			    Tk_Uid fg, Tk_Uid bg));
EXTERN int		Tk_GetDoublePixels _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, CONST char *string,
			    double *doublePtr));
EXTERN Tk_Font		Tk_GetFont _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, CONST char *string));
EXTERN Tk_Font		Tk_GetFontFromObj _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, Tcl_Obj *objPtr));
EXTERN void		Tk_GetFontMetrics _ANSI_ARGS_((Tk_Font font,
			    Tk_FontMetrics *fmPtr));
EXTERN GC		Tk_GetGC _ANSI_ARGS_((Tk_Window tkwin,
			    unsigned long valueMask, XGCValues *valuePtr));
EXTERN Tk_Image		Tk_GetImage _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, char *name,
			    Tk_ImageChangedProc *changeProc,
			    ClientData clientData));
EXTERN ClientData	Tk_GetImageMasterData _ANSI_ARGS_ ((Tcl_Interp *interp,
			    char *name, Tk_ImageType **typePtrPtr));
EXTERN Tk_ItemType *	Tk_GetItemTypes _ANSI_ARGS_((void));
EXTERN int		Tk_GetJoinStyle _ANSI_ARGS_((Tcl_Interp *interp,
			    char *string, int *joinPtr));
EXTERN int		Tk_GetJustify _ANSI_ARGS_((Tcl_Interp *interp,
			    char *string, Tk_Justify *justifyPtr));
EXTERN int		Tk_GetNumMainWindows _ANSI_ARGS_((void));
EXTERN Tk_Uid		Tk_GetOption _ANSI_ARGS_((Tk_Window tkwin, char *name,
			    char *className));
EXTERN int		Tk_GetPixels _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, char *string, int *intPtr));
EXTERN Pixmap		Tk_GetPixmap _ANSI_ARGS_((Display *display, Drawable d,
			    int width, int height, int depth));
EXTERN int		Tk_GetRelief _ANSI_ARGS_((Tcl_Interp *interp,
			    char *name, int *reliefPtr));
EXTERN void		Tk_GetRootCoords _ANSI_ARGS_ ((Tk_Window tkwin,
			    int *xPtr, int *yPtr));
EXTERN int		Tk_GetScrollInfo _ANSI_ARGS_((Tcl_Interp *interp,
			    int argc, char **argv, double *dblPtr,
			    int *intPtr));
EXTERN int		Tk_GetScreenMM _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, char *string, double *doublePtr));
EXTERN int		Tk_GetSelection _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, Atom selection, Atom target,
			    Tk_GetSelProc *proc, ClientData clientData));
EXTERN Tk_Uid		Tk_GetUid _ANSI_ARGS_((CONST char *string));
EXTERN Visual *		Tk_GetVisual _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, char *string, int *depthPtr,
			    Colormap *colormapPtr));
EXTERN void		Tk_GetVRootGeometry _ANSI_ARGS_((Tk_Window tkwin,
			    int *xPtr, int *yPtr, int *widthPtr,
			    int *heightPtr));
EXTERN int		Tk_Grab _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, int grabGlobal));
EXTERN void		Tk_HandleEvent _ANSI_ARGS_((XEvent *eventPtr));
EXTERN Tk_Window      	Tk_IdToWindow _ANSI_ARGS_((Display *display,
			    Window window));
EXTERN void		Tk_ImageChanged _ANSI_ARGS_((
			    Tk_ImageMaster master, int x, int y,
			    int width, int height, int imageWidth,
			    int imageHeight));
EXTERN int		Tk_Init _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN int		Tk_InitStandAlone _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN Atom		Tk_InternAtom _ANSI_ARGS_((Tk_Window tkwin,
			    char *name));
EXTERN int		Tk_IntersectTextLayout _ANSI_ARGS_((
			    Tk_TextLayout layout, int x, int y, int width,
			    int height));
EXTERN void		Tk_Main _ANSI_ARGS_((int argc, char **argv,
			    Tcl_AppInitProc *appInitProc));
#define Tk_Main(argc, argv, appInitProc) \
	TkMain(argc, argv, appInitProc, Tcl_CreateInterp())
EXTERN void		TkMain _ANSI_ARGS_((int argc, char **argv,
			    Tcl_AppInitProc *appInitProc, Tcl_Interp *interp));
EXTERN void		Tk_MainLoop _ANSI_ARGS_((void));
EXTERN void		Tk_MaintainGeometry _ANSI_ARGS_((Tk_Window slave,
			    Tk_Window master, int x, int y, int width,
			    int height));
EXTERN Tk_Window	Tk_MainWindow _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN void		Tk_MakeWindowExist _ANSI_ARGS_((Tk_Window tkwin));
EXTERN void		Tk_ManageGeometry _ANSI_ARGS_((Tk_Window tkwin,
			    Tk_GeomMgr *mgrPtr, ClientData clientData));
EXTERN void		Tk_MapWindow _ANSI_ARGS_((Tk_Window tkwin));
EXTERN int		Tk_MeasureChars _ANSI_ARGS_((Tk_Font tkfont,
			    CONST char *source, int maxChars, int maxPixels,
			    int flags, int *lengthPtr));
EXTERN void		Tk_MoveResizeWindow _ANSI_ARGS_((Tk_Window tkwin,
			    int x, int y, int width, int height));
EXTERN void		Tk_MoveWindow _ANSI_ARGS_((Tk_Window tkwin, int x,
			    int y));
EXTERN void		Tk_MoveToplevelWindow _ANSI_ARGS_((Tk_Window tkwin,
			    int x, int y));
EXTERN char *		Tk_NameOf3DBorder _ANSI_ARGS_((Tk_3DBorder border));
EXTERN char *		Tk_NameOfAnchor _ANSI_ARGS_((Tk_Anchor anchor));
EXTERN char *		Tk_NameOfBitmap _ANSI_ARGS_((Display *display,
			    Pixmap bitmap));
EXTERN char *		Tk_NameOfCapStyle _ANSI_ARGS_((int cap));
EXTERN char *		Tk_NameOfColor _ANSI_ARGS_((XColor *colorPtr));
EXTERN char *		Tk_NameOfCursor _ANSI_ARGS_((Display *display,
			    Tk_Cursor cursor));
EXTERN char *		Tk_NameOfFont _ANSI_ARGS_((Tk_Font font));
EXTERN char *		Tk_NameOfImage _ANSI_ARGS_((
			    Tk_ImageMaster imageMaster));
EXTERN char *		Tk_NameOfJoinStyle _ANSI_ARGS_((int join));
EXTERN char *		Tk_NameOfJustify _ANSI_ARGS_((Tk_Justify justify));
EXTERN char *		Tk_NameOfRelief _ANSI_ARGS_((int relief));
EXTERN Tk_Window	Tk_NameToWindow _ANSI_ARGS_((Tcl_Interp *interp,
			    char *pathName, Tk_Window tkwin));
EXTERN void		Tk_OwnSelection _ANSI_ARGS_((Tk_Window tkwin,
			    Atom selection, Tk_LostSelProc *proc,
			    ClientData clientData));
EXTERN int		Tk_ParseArgv _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, int *argcPtr, char **argv,
			    Tk_ArgvInfo *argTable, int flags));
EXTERN int		Tk_OrientParseProc _ANSI_ARGS_((
			    ClientData clientData, Tcl_Interp *interp,
			    Tk_Window tkwin, CONST char *value,
			    char *widgRec, int offset));
EXTERN char *		Tk_OrientPrintProc _ANSI_ARGS_((
			    ClientData clientData, Tk_Window tkwin,
			    char *widgRec, int offset,
			    Tcl_FreeProc **freeProcPtr));
EXTERN void		Tk_PhotoPutBlock _ANSI_ARGS_((Tk_PhotoHandle handle,
			    Tk_PhotoImageBlock *blockPtr, int x, int y,
			    int width, int height));
EXTERN void		Tk_PhotoPutZoomedBlock _ANSI_ARGS_((
			    Tk_PhotoHandle handle,
			    Tk_PhotoImageBlock *blockPtr, int x, int y,
			    int width, int height, int zoomX, int zoomY,
			    int subsampleX, int subsampleY));
EXTERN int		Tk_PhotoGetImage _ANSI_ARGS_((Tk_PhotoHandle handle,
			    Tk_PhotoImageBlock *blockPtr));
EXTERN void		Tk_PhotoBlank _ANSI_ARGS_((Tk_PhotoHandle handle));
EXTERN void		Tk_PhotoExpand _ANSI_ARGS_((Tk_PhotoHandle handle,
			    int width, int height ));
EXTERN void		Tk_PhotoGetSize _ANSI_ARGS_((Tk_PhotoHandle handle,
			    int *widthPtr, int *heightPtr));
EXTERN void		Tk_PhotoSetSize _ANSI_ARGS_((Tk_PhotoHandle handle,
			    int width, int height));
EXTERN int		Tk_PointToChar _ANSI_ARGS_((Tk_TextLayout layout,
			    int x, int y));
EXTERN int		Tk_PostscriptBitmap _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, Tk_PostscriptInfo psInfo,
			    Pixmap bitmap, int startX, int startY,
			    int width, int height));
EXTERN int		Tk_PostscriptColor _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_PostscriptInfo psInfo, XColor *colorPtr));
EXTERN int		Tk_PostscriptFontName _ANSI_ARGS_((Tk_Font tkfont,
			    Tcl_DString *dsPtr));
EXTERN int		Tk_PostscriptFont _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_PostscriptInfo psInfo, Tk_Font font));
EXTERN int		Tk_PostscriptImage _ANSI_ARGS_((Tk_Image image,
			    Tcl_Interp *interp, Tk_Window tkwin,
			    Tk_PostscriptInfo psinfo, int x, int y,
			    int width, int height, int prepass));
EXTERN void		Tk_PostscriptPath _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_PostscriptInfo psInfo, double *coordPtr,
			    int numPoints));
EXTERN int		Tk_PostscriptStipple _ANSI_ARGS_((Tcl_Interp *interp,
			    Tk_Window tkwin, Tk_PostscriptInfo psInfo,
			    Pixmap bitmap));
EXTERN double		Tk_PostscriptY _ANSI_ARGS_((double y,
			    Tk_PostscriptInfo psInfo));
EXTERN void		Tk_PreserveColormap _ANSI_ARGS_((Display *display,
			    Colormap colormap));
EXTERN void		Tk_QueueWindowEvent _ANSI_ARGS_((XEvent *eventPtr,
			    Tcl_QueuePosition position));
EXTERN void		Tk_RedrawImage _ANSI_ARGS_((Tk_Image image, int imageX,
			    int imageY, int width, int height,
			    Drawable drawable, int drawableX, int drawableY));
EXTERN void		Tk_ResizeWindow _ANSI_ARGS_((Tk_Window tkwin,
			    int width, int height));
EXTERN int		Tk_RestackWindow _ANSI_ARGS_((Tk_Window tkwin,
			    int aboveBelow, Tk_Window other));
EXTERN Tk_RestrictProc *Tk_RestrictEvents _ANSI_ARGS_((Tk_RestrictProc *proc,
			    ClientData arg, ClientData *prevArgPtr));
EXTERN int		Tk_SafeInit _ANSI_ARGS_((Tcl_Interp *interp));
EXTERN char *		Tk_SetAppName _ANSI_ARGS_((Tk_Window tkwin,
			    char *name));
EXTERN void		Tk_SetBackgroundFromBorder _ANSI_ARGS_((
			    Tk_Window tkwin, Tk_3DBorder border));
EXTERN void		Tk_SetClass _ANSI_ARGS_((Tk_Window tkwin,
			    char *className));
EXTERN void		Tk_SetGrid _ANSI_ARGS_((Tk_Window tkwin,
			    int reqWidth, int reqHeight, int gridWidth,
			    int gridHeight));
EXTERN void		Tk_SetInternalBorder _ANSI_ARGS_((Tk_Window tkwin,
			    int width));
EXTERN void		Tk_SetWindowBackground _ANSI_ARGS_((Tk_Window tkwin,
			    unsigned long pixel));
EXTERN void		Tk_SetWindowBackgroundPixmap _ANSI_ARGS_((
			    Tk_Window tkwin, Pixmap pixmap));
EXTERN void		Tk_SetWindowBorder _ANSI_ARGS_((Tk_Window tkwin,
			    unsigned long pixel));
EXTERN void		Tk_SetWindowBorderWidth _ANSI_ARGS_((Tk_Window tkwin,
			    int width));
EXTERN void		Tk_SetWindowBorderPixmap _ANSI_ARGS_((Tk_Window tkwin,
			    Pixmap pixmap));
EXTERN void		Tk_SetWindowColormap _ANSI_ARGS_((Tk_Window tkwin,
			    Colormap colormap));
EXTERN int		Tk_SetWindowVisual _ANSI_ARGS_((Tk_Window tkwin,
			    Visual *visual, int depth,
			    Colormap colormap));
EXTERN void		Tk_SizeOfBitmap _ANSI_ARGS_((Display *display,
			    Pixmap bitmap, int *widthPtr,
			    int *heightPtr));
EXTERN void		Tk_SizeOfImage _ANSI_ARGS_((Tk_Image image,
			    int *widthPtr, int *heightPtr));
EXTERN int		Tk_StateParseProc _ANSI_ARGS_((
			    ClientData clientData, Tcl_Interp *interp,
			    Tk_Window tkwin, CONST char *value,
			    char *widgRec, int offset));
EXTERN char *		Tk_StatePrintProc _ANSI_ARGS_((
			    ClientData clientData, Tk_Window tkwin,
			    char *widgRec, int offset,
			    Tcl_FreeProc **freeProcPtr));
EXTERN int		Tk_StrictMotif _ANSI_ARGS_((Tk_Window tkwin));
EXTERN void		Tk_TextLayoutToPostscript _ANSI_ARGS_((
			    Tcl_Interp *interp, Tk_TextLayout layout));
EXTERN int		Tk_TextWidth _ANSI_ARGS_((Tk_Font font,
			    CONST char *string, int numChars));
EXTERN void		Tk_UndefineCursor _ANSI_ARGS_((Tk_Window window));
EXTERN void		Tk_UnderlineChars _ANSI_ARGS_((Display *display,
			    Drawable drawable, GC gc, Tk_Font tkfont,
			    CONST char *source, int x, int y, int firstChar,
			    int lastChar));
EXTERN void		Tk_UnderlineTextLayout _ANSI_ARGS_((
			    Display *display, Drawable drawable, GC gc,
			    Tk_TextLayout layout, int x, int y,
			    int underline));
EXTERN void		Tk_Ungrab _ANSI_ARGS_((Tk_Window tkwin));
EXTERN void		Tk_UnmaintainGeometry _ANSI_ARGS_((Tk_Window slave,
			    Tk_Window master));
EXTERN void		Tk_UnmapWindow _ANSI_ARGS_((Tk_Window tkwin));
EXTERN void		Tk_UnsetGrid _ANSI_ARGS_((Tk_Window tkwin));
EXTERN void		Tk_UpdatePointer _ANSI_ARGS_((Tk_Window tkwin,
			    int x, int y, int state));

/*
 * Tcl commands exported by Tk:
 */

EXTERN int		Tk_BellObjCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int objc,
			    Tcl_Obj *CONST objv[]));
EXTERN int		Tk_BindCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_BindtagsCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_ButtonCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_CanvasObjCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, Tcl_Obj *CONST objv[]));
EXTERN int		Tk_CheckbuttonCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_ClipboardCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int              Tk_ChooseColorCmd _ANSI_ARGS_((ClientData clientData,
                            Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_DdeObjCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, Tcl_Obj *CONST argv[]));
EXTERN int		Tk_DestroyCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_EntryCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_EventObjCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, Tcl_Obj *CONST argv[]));
EXTERN int		Tk_FrameCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_FocusObjCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, Tcl_Obj *CONST objv[]));
EXTERN int		Tk_FontObjCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int objc,
			    Tcl_Obj *CONST objv[]));
EXTERN int              Tk_GetOpenFileCmd _ANSI_ARGS_((ClientData clientData,
                            Tcl_Interp *interp, int argc, char **argv));
EXTERN int              Tk_GetSaveFileCmd _ANSI_ARGS_((ClientData clientData,
                            Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_GrabCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_GridCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_ImageObjCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, Tcl_Obj *CONST objv[]));
EXTERN int		Tk_LabelCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_ListboxCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_LowerCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_MenuCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_MenubuttonCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int              Tk_MessageBoxCmd _ANSI_ARGS_((ClientData clientData,
                            Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_MessageCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_OptionCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_PackCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_PlaceCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_RadiobuttonCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_RaiseCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_ScaleCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_ScrollbarCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_SelectionCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_SendCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_SendObjCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, Tcl_Obj *CONST argv[]));
EXTERN int		Tk_TextCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_TkObjCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int objc,
			    Tcl_Obj *CONST objv[]));
EXTERN int		Tk_TkwaitCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_ToplevelCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_UpdateCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));
EXTERN int		Tk_WinfoObjCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int objc,
			    Tcl_Obj *CONST objv[]));
EXTERN int		Tk_WmCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int argc, char **argv));

typedef struct Tk_StubTable {
    int magic;
    struct Tk_StubTableHooks *hooks; /* reserved for future expansion */
    void 		(*mainLoop) _ANSI_ARGS_((void));
    XColor *		(*d3BorderColor) _ANSI_ARGS_((Tk_3DBorder));
    GC			(*d3BorderGC) _ANSI_ARGS_((Tk_Window, Tk_3DBorder, 
			    int));
    void		(*addOption) _ANSI_ARGS_((Tk_Window, char *, char *, 
			    int));
    void		(*bindEvent) _ANSI_ARGS_((Tk_BindingTable, XEvent *, 
			    Tk_Window, int, ClientData *));
    void		(*canvasDrawableCoords) _ANSI_ARGS_((Tk_Canvas, double, 
			    double, short *, short *));
    void		(*canvasEventuallyRedraw) _ANSI_ARGS_((Tk_Canvas, int, 
			    int, int, int));
    int			(*canvasGetCoord) _ANSI_ARGS_((Tcl_Interp *, Tk_Canvas, 
			    char *, double *));
    Tk_CanvasTextInfo *	(*canvasGetTextInfo) _ANSI_ARGS_((Tk_Canvas));
    int			(*canvasPsBitmap) _ANSI_ARGS_((Tcl_Interp *, Tk_Canvas, 
			    Pixmap, int, int, int, int));
    int			(*canvasPsColor) _ANSI_ARGS_((Tcl_Interp *, Tk_Canvas, 
			    XColor *));
    int			(*canvasPsFont) _ANSI_ARGS_((Tcl_Interp *, Tk_Canvas, 
			    Tk_Font));
    void		(*canvasPsPath) _ANSI_ARGS_((Tcl_Interp *, Tk_Canvas, 
			    double *, int));
    int			(*canvasPsStipple) _ANSI_ARGS_((Tcl_Interp *, 
			    Tk_Canvas, Pixmap));
    double		(*canvasPsY) _ANSI_ARGS_((Tk_Canvas, double));
    void		(*canvasSetStippleOrigin) _ANSI_ARGS_((Tk_Canvas, GC));
    int			(*canvasTagsParseProc) _ANSI_ARGS_((ClientData, 
			    Tcl_Interp *, Tk_Window, char *, char *, int));
    char *		(*canvasTagsPrintProc) _ANSI_ARGS_((ClientData, 
			    Tk_Window, char *, int, Tcl_FreeProc **));
    Tk_Window		(*canvasTkwin) _ANSI_ARGS_((Tk_Canvas));
    void		(*canvasWindowCoords) _ANSI_ARGS_((Tk_Canvas, double, 
			    double, short *, short *));
    void		(*changeWindowAttributes) _ANSI_ARGS_((Tk_Window, 
			    unsigned long, XSetWindowAttributes *));
    int			(*charBbox) _ANSI_ARGS_((Tk_TextLayout, int, int *, 
			    int *, int *, int *));
    void		(*clearSelection) _ANSI_ARGS_((Tk_Window, Atom));
    int			(*clipboardAppend) _ANSI_ARGS_((Tcl_Interp *, 
			    Tk_Window, Atom, Atom, char*));
    int			(*clipboardClear) _ANSI_ARGS_((Tcl_Interp *, 
			    Tk_Window));
    int			(*configureInfo) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    Tk_ConfigSpec *, char *, char *, int));
    int			(*configureValue) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    Tk_ConfigSpec *, char *, char *, int));
    int			(*configureWidget) _ANSI_ARGS_((Tcl_Interp *, 
			    Tk_Window, Tk_ConfigSpec *, int, char **, char *, 
			    int));
    void		(*configureWindow) _ANSI_ARGS_((Tk_Window, 
			    unsigned int, XWindowChanges *));
    Tk_TextLayout	(*computeTextLayout) _ANSI_ARGS_((Tk_Font, 
			    CONST char *, int, int, Tk_Justify, int, int *, 
			    int *));
    Tk_Window		(*coordsToWindow) _ANSI_ARGS_((int, int, Tk_Window));
    unsigned long	(*createBinding) _ANSI_ARGS_((Tcl_Interp *, 
			    Tk_BindingTable, ClientData, char *, char *, int));
    Tk_BindingTable	(*createBindingTable) _ANSI_ARGS_((Tcl_Interp *));
    Tk_ErrorHandler	(*createErrorHandler) _ANSI_ARGS_((Display *, int, int, 
			    int, Tk_ErrorProc *, ClientData));
    void		(*createEventHandler) _ANSI_ARGS_((Tk_Window, 
			    unsigned long, Tk_EventProc *, ClientData));
    void		(*createGenericHandler) _ANSI_ARGS_((Tk_GenericProc *, 
			    ClientData));
    void		(*createImageType) _ANSI_ARGS_((Tk_ImageType *));
    void		(*createItemType) _ANSI_ARGS_((Tk_ItemType *));
    void		(*createPhotoImageFormat) _ANSI_ARGS_((
			    Tk_PhotoImageFormat *));
    void		(*createSelHandler) _ANSI_ARGS_((Tk_Window, Atom, Atom, 
			    Tk_SelectionProc *, ClientData, Atom));
    Tk_Window		(*createWindow) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    char *, char *));
    Tk_Window		(*createWindowFromPath) _ANSI_ARGS_((Tcl_Interp *, 
			    Tk_Window, char *, char *));
    int			(*defineBitmap) _ANSI_ARGS_((Tcl_Interp *, Tk_Uid, 
			    char *, int, int));
    void		(*defineCursor) _ANSI_ARGS_((Tk_Window, Tk_Cursor));
    void		(*deleteAllBindings) _ANSI_ARGS_((Tk_BindingTable, 
			    ClientData));
    int			(*deleteBinding) _ANSI_ARGS_((Tcl_Interp *, 
			    Tk_BindingTable, ClientData, char *));
    void		(*deleteBindingTable) _ANSI_ARGS_((Tk_BindingTable));
    void		(*deleteErrorHandler) _ANSI_ARGS_((Tk_ErrorHandler));
    void		(*deleteEventHandler) _ANSI_ARGS_((Tk_Window, 
			    unsigned long, Tk_EventProc *, ClientData));
    void		(*deleteGenericHandler) _ANSI_ARGS_((Tk_GenericProc *, 
			    ClientData));
    void		(*deleteImage) _ANSI_ARGS_((Tcl_Interp *, char *));
    void		(*deleteSelHandler) _ANSI_ARGS_((Tk_Window, Atom, 
			    Atom));
    void		(*destroyWindow) _ANSI_ARGS_((Tk_Window));
    char *		(*displayName) _ANSI_ARGS_((Tk_Window));
    int			(*distanceToTextLayout) _ANSI_ARGS_((Tk_TextLayout, 
			    int, int));
    void		(*draw3DPolygon) _ANSI_ARGS_((Tk_Window, Drawable, 
			    Tk_3DBorder, XPoint *, int, int, int));
    void		(*draw3DRectangle) _ANSI_ARGS_((Tk_Window, Drawable, 
			    Tk_3DBorder, int, int, int, int, int, int));
    void		(*drawChars) _ANSI_ARGS_((Display *, Drawable, GC, 
			    Tk_Font, CONST char *, int, int, int));
    void		(*drawFocusHighlight) _ANSI_ARGS_((Tk_Window, GC, int, 
			    Drawable));
    void		(*drawTextLayout) _ANSI_ARGS_((Display *, Drawable, GC, 
			    Tk_TextLayout, int, int, int, int));
    void		(*fill3DPolygon) _ANSI_ARGS_((Tk_Window, Drawable, 
			    Tk_3DBorder, XPoint *, int, int, int));
    void		(*fill3DRectangle) _ANSI_ARGS_((Tk_Window, Drawable, 
			    Tk_3DBorder, int, int, int, int, int, int));
    Tk_PhotoHandle	(*findPhoto) _ANSI_ARGS_((Tcl_Interp *, char *));
    Font		(*fontId) _ANSI_ARGS_((Tk_Font));
    void		(*free3DBorder) _ANSI_ARGS_((Tk_3DBorder));
    void		(*freeBitmap) _ANSI_ARGS_((Display *, Pixmap));
    void		(*freeColor) _ANSI_ARGS_((XColor *));
    void		(*freeColormap) _ANSI_ARGS_((Display *, Colormap));
    void		(*freeCursor) _ANSI_ARGS_((Display *, Tk_Cursor));
    void		(*freeFont) _ANSI_ARGS_((Tk_Font));
    void		(*freeGC) _ANSI_ARGS_((Display *, GC));
    void		(*freeImage) _ANSI_ARGS_((Tk_Image));
    void		(*freeOptions) _ANSI_ARGS_((Tk_ConfigSpec *, char *, 
			    Display *, int));
    void		(*freePixmap) _ANSI_ARGS_((Display *, Pixmap));
    void		(*freeTextLayout) _ANSI_ARGS_((Tk_TextLayout));
    GC			(*gCForColor) _ANSI_ARGS_((XColor *, Drawable));
    void		(*geometryRequest) _ANSI_ARGS_((Tk_Window, int, int));
    Tk_3DBorder		(*get3DBorder) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    Tk_Uid));
    void		(*getAllBindings) _ANSI_ARGS_((Tcl_Interp *, 
			    Tk_BindingTable, ClientData));
    int			(*getAnchor) _ANSI_ARGS_((Tcl_Interp *, char *, 
			    Tk_Anchor *));
    char *		(*getAtomName) _ANSI_ARGS_((Tk_Window, Atom));
    char *		(*getBinding) _ANSI_ARGS_((Tcl_Interp *, 
			    Tk_BindingTable, ClientData, char *));
    Pixmap		(*getBitmap) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    Tk_Uid));
    Pixmap		(*getBitmapFromData) _ANSI_ARGS_((Tcl_Interp *, 
			    Tk_Window, char *, int, int));
    int			(*getCapStyle) _ANSI_ARGS_((Tcl_Interp *, char *, 
			    int *));
    XColor *		(*getColor) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    Tk_Uid));
    XColor *		(*getColorByValue) _ANSI_ARGS_((Tk_Window, XColor *));
    Colormap		(*getColormap) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    char *));
    Tk_Cursor		(*getCursor) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    Tk_Uid));
    Tk_Cursor		(*getCursorFromData) _ANSI_ARGS_((Tcl_Interp *, 
			    Tk_Window, char *, char *, int, int, int, int, 
			    Tk_Uid, Tk_Uid));
    Tk_Font		(*getFont) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    CONST char *));
    Tk_Font		(*getFontFromObj) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    Tcl_Obj *));
    void		(*getFontMetrics) _ANSI_ARGS_((Tk_Font, 
			    Tk_FontMetrics *));
    GC			(*getGC) _ANSI_ARGS_((Tk_Window, unsigned long, 
			    XGCValues *));
    Tk_Image		(*getImage) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    char *, Tk_ImageChangedProc *, ClientData));
    ClientData		(*getImageMasterData) _ANSI_ARGS_((Tcl_Interp *, 
			    char *, Tk_ImageType **));
    Tk_ItemType *	(*getItemTypes) _ANSI_ARGS_((void));
    int			(*getJoinStyle) _ANSI_ARGS_((Tcl_Interp *, char *, 
			    int *));
    int			(*getJustify) _ANSI_ARGS_((Tcl_Interp *, char *, 
			    Tk_Justify *));
    int			(*getNumMainWindows) _ANSI_ARGS_((void));
    Tk_Uid		(*getOption) _ANSI_ARGS_((Tk_Window, char *, char *));
    int			(*getPixels) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    char *, int *));
    Pixmap		(*getPixmap) _ANSI_ARGS_((Display *, Drawable, int, 
			    int, int));
    int			(*getRelief) _ANSI_ARGS_((Tcl_Interp *, char *, int *));
    void		(*getRootCoords) _ANSI_ARGS_((Tk_Window, int *, int *));
    int			(*getScrollInfo) _ANSI_ARGS_((Tcl_Interp *, int, 
			    char **, double *, int *));
    int			(*getScreenMM) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    char *, double *));
    int			(*getSelection) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    Atom, Atom, Tk_GetSelProc *, ClientData));
    Tk_Uid		(*getUid) _ANSI_ARGS_((CONST char *));
    Visual *		(*getVisual) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    char *, int *, Colormap *));
    void		(*getVRootGeometry) _ANSI_ARGS_((Tk_Window, int *, 
			    int *, int *, int *));
    int			(*grab) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, int));
    void		(*handleEvent) _ANSI_ARGS_((XEvent *));
    Tk_Window		(*idToWindow) _ANSI_ARGS_((Display *, Window));
    void		(*imageChanged) _ANSI_ARGS_((Tk_ImageMaster, int, int, 
			    int, int, int, int));
    int			(*init) _ANSI_ARGS_((Tcl_Interp *));
    Atom		(*internAtom) _ANSI_ARGS_((Tk_Window, char *));
    int			(*intersectTextLayout) _ANSI_ARGS_((Tk_TextLayout, int, 
			    int, int, int));
    void		(*tkmain) _ANSI_ARGS_((int, char **, Tcl_AppInitProc *, 
			    Tcl_Interp *));
    void		(*maintainGeometry) _ANSI_ARGS_((Tk_Window, Tk_Window, 
			    int, int, int, int));
    Tk_Window		(*mainWindow) _ANSI_ARGS_((Tcl_Interp *));
    void		(*makeWindowExist) _ANSI_ARGS_((Tk_Window));
    void		(*manageGeometry) _ANSI_ARGS_((Tk_Window, Tk_GeomMgr *, 
			    ClientData));
    void		(*mapWindow) _ANSI_ARGS_((Tk_Window));
    int			(*measureChars) _ANSI_ARGS_((Tk_Font, CONST char *, 
			    int, int, int, int *));
    void		(*moveResizeWindow) _ANSI_ARGS_((Tk_Window, int, int, 
			    int, int));
    void		(*moveWindow) _ANSI_ARGS_((Tk_Window, int, int));
    void		(*moveToplevelWindow) _ANSI_ARGS_((Tk_Window, int, 
			    int));
    char *		(*nameOf3DBorder) _ANSI_ARGS_((Tk_3DBorder));
    char *		(*nameOfAnchor) _ANSI_ARGS_((Tk_Anchor));
    char *		(*nameOfBitmap) _ANSI_ARGS_((Display *, Pixmap));
    char *		(*nameOfCapStyle) _ANSI_ARGS_((int));
    char *		(*nameOfColor) _ANSI_ARGS_((XColor *));
    char *		(*nameOfCursor) _ANSI_ARGS_((Display *, Tk_Cursor));
    char *		(*nameOfFont) _ANSI_ARGS_((Tk_Font));
    char *		(*nameOfImage) _ANSI_ARGS_((Tk_ImageMaster));
    char *		(*nameOfJoinStyle) _ANSI_ARGS_((int));
    char *		(*nameOfJustify) _ANSI_ARGS_((Tk_Justify));
    char *		(*nameOfRelief) _ANSI_ARGS_((int));
    Tk_Window		(*nameToWindow) _ANSI_ARGS_((Tcl_Interp *, char *, 
			    Tk_Window));
    void		(*ownSelection) _ANSI_ARGS_((Tk_Window, Atom, 
			    Tk_LostSelProc *, ClientData));
    int			(*parseArgv) _ANSI_ARGS_((Tcl_Interp *, Tk_Window, 
			    int *, char **, Tk_ArgvInfo *, int));
    void		(*photoPutBlock) _ANSI_ARGS_((Tk_PhotoHandle, 
			    Tk_PhotoImageBlock *, int, int, int, int));
    void		(*photoPutZoomedBlock) _ANSI_ARGS_((Tk_PhotoHandle, 
			    Tk_PhotoImageBlock *, int, int, int, int, int, int, 
			    int, int));
    int			(*photoGetImage) _ANSI_ARGS_((Tk_PhotoHandle, 
			    Tk_PhotoImageBlock *));
    void		(*photoBlank) _ANSI_ARGS_((Tk_PhotoHandle));
    void		(*photoExpand) _ANSI_ARGS_((Tk_PhotoHandle, int, int));
    void		(*photoGetSize) _ANSI_ARGS_((Tk_PhotoHandle, int *, 
			    int *));
    void		(*photoSetSize) _ANSI_ARGS_((Tk_PhotoHandle, int, int));
    int			(*pointToChar) _ANSI_ARGS_((Tk_TextLayout, int, int));
    int			(*postscriptFontName) _ANSI_ARGS_((Tk_Font, 
			    Tcl_DString *));
    void		(*preserveColormap) _ANSI_ARGS_((Display *, Colormap));
    void		(*queueWindowEvent) _ANSI_ARGS_((XEvent *, 
			    Tcl_QueuePosition));
    void		(*redrawImage) _ANSI_ARGS_((Tk_Image, int, int, int, 
			    int, Drawable, int, int));
    void		(*resizeWindow) _ANSI_ARGS_((Tk_Window, int, int));
    int			(*restackWindow) _ANSI_ARGS_((Tk_Window, int, 
			    Tk_Window));
    Tk_RestrictProc *	(*restrictEvents) _ANSI_ARGS_((Tk_RestrictProc *, 
			    ClientData, ClientData *));
    int			(*safeInit) _ANSI_ARGS_((Tcl_Interp *));
    char *		(*setAppName) _ANSI_ARGS_((Tk_Window, char *));
    void		(*setBackgroundFromBorder) _ANSI_ARGS_((Tk_Window, 
			    Tk_3DBorder));
    void		(*setClass) _ANSI_ARGS_((Tk_Window, char *));
    void		(*setGrid) _ANSI_ARGS_((Tk_Window, int, int, int, int));
    void		(*setInternalBorder) _ANSI_ARGS_((Tk_Window, int));
    void		(*setWindowBackground) _ANSI_ARGS_((Tk_Window, 
			    unsigned long));
    void		(*setWindowBackgroundPixmap) _ANSI_ARGS_((Tk_Window, 
			    Pixmap));
    void		(*setWindowBorder) _ANSI_ARGS_((Tk_Window, 
			    unsigned long));
    void		(*setWindowBorderWidth) _ANSI_ARGS_((Tk_Window, int));
    void		(*setWindowBorderPixmap) _ANSI_ARGS_((Tk_Window, 
			    Pixmap));
    void		(*setWindowColormap) _ANSI_ARGS_((Tk_Window, Colormap));
    int			(*setWindowVisual) _ANSI_ARGS_((Tk_Window, Visual *, 
			    int, Colormap));
    void		(*sizeOfBitmap) _ANSI_ARGS_((Display *, Pixmap, int *, 
			    int *));
    void		(*sizeOfImage) _ANSI_ARGS_((Tk_Image, int *, int *));
    int			(*strictMotif) _ANSI_ARGS_((Tk_Window));
    void		(*textLayoutToPostscript) _ANSI_ARGS_((Tcl_Interp *, 
			    Tk_TextLayout));
    int			(*textWidth) _ANSI_ARGS_((Tk_Font, CONST char *, int));
    void		(*undefineCursor) _ANSI_ARGS_((Tk_Window));
    void		(*underlineChars) _ANSI_ARGS_((Display *, Drawable, GC, 
			    Tk_Font, CONST char *, int, int, int, int));
    void		(*underlineTextLayout) _ANSI_ARGS_((Display *, 
			    Drawable, GC, Tk_TextLayout, int, int, int));
    void		(*ungrab) _ANSI_ARGS_((Tk_Window));
    void		(*unmaintainGeometry) _ANSI_ARGS_((Tk_Window, 
			    Tk_Window));
    void		(*unmapWindow) _ANSI_ARGS_((Tk_Window));
    void		(*unsetGrid) _ANSI_ARGS_((Tk_Window));
} Tk_StubTable;

#ifdef TK_USE_STUBS

extern Tk_StubTable *tkStubTable;

#define Tk_MainLoop() \
	(tkStubTable->mainLoop())
#define Tk_3DBorderColor(border) \
	(tkStubTable->d3BorderColor)(border)
#define Tk_3DBorderGC(tkwin, border, which) \
	(tkStubTable->d3BorderGC)(tkwin, border, which)
#define Tk_AddOption(tkwin, name, value, priority) \
	(tkStubTable->addOption)(tkwin, name, value, priority)
#define Tk_BindEvent(bindingTable, eventPtr, tkwin, numObjects, objectPtr) \
	(tkStubTable->bindEvent)(bindingTable, eventPtr, tkwin, numObjects, \
	objectPtr)
#define Tk_CanvasDrawableCoords(canvas, x, y, drawableXPtr, drawableYPtr) \
	(tkStubTable->canvasDrawableCoords)(canvas, x, y, drawableXPtr, \
	drawableYPtr)
#define Tk_CanvasEventuallyRedraw(canvas, x1, y1, x2, y2) \
	(tkStubTable->canvasEventuallyRedraw)(canvas, x1, y1, x2, y2)
#define Tk_CanvasGetCoord(interp, canvas, string, doublePtr) \
	(tkStubTable->canvasGetCoord)(interp, canvas, string, doublePtr)
#define Tk_CanvasGetTextInfo(canvas) \
	(tkStubTable->canvasGetTextInfo)(canvas)
#define Tk_CanvasPsBitmap(interp, canvas, bitmap, x, y, width, height) \
	(tkStubTable->canvasPsBitmap)(interp, canvas, bitmap, x, y, width, \
	height)
#define Tk_CanvasPsColor(interp, canvas, colorPtr) \
	(tkStubTable->canvasPsColor)(interp, canvas, colorPtr)
#define Tk_CanvasPsFont(interp, canvas, font) \
	(tkStubTable->canvasPsFont)(interp, canvas, font)
#define Tk_CanvasPsPath(interp, canvas, coordPtr, numPoints) \
	(tkStubTable->canvasPsPath)(interp, canvas, coordPtr, numPoints)
#define Tk_CanvasPsStipple(interp, canvas, bitmap) \
	(tkStubTable->canvasPsStipple)(interp, canvas, bitmap)
#define Tk_CanvasPsY(canvas, y) \
	(tkStubTable->canvasPsY)(canvas, y)
#define Tk_CanvasSetStippleOrigin(canvas, gc) \
	(tkStubTable->canvasSetStippleOrigin)(canvas, gc)
#define Tk_CanvasTagsParseProc(clientData, interp, tkwin, value, widgRec, \
	offset) \
	(tkStubTable->canvasTagsParseProc)(clientData, interp, tkwin, value, \
	widgRec, offset)
#define Tk_CanvasTagsPrintProc(clientData, tkwin, widgRec, offset, \
	freeProcPtr) \
	(tkStubTable->canvasTagsPrintProc)(clientData, tkwin, widgRec, \
	offset, freeProcPtr)
#define Tk_CanvasTkwin(canvas) \
	(tkStubTable->canvasTkwin)(canvas)
#define Tk_CanvasWindowCoords(canvas, x, y, screenXPtr, screenYPtr) \
	(tkStubTable->canvasWindowCoords)(canvas, x, y, screenXPtr, \
	screenYPtr)
#define Tk_ChangeWindowAttributes(tkwin, valueMask, attsPtr) \
	(tkStubTable->changeWindowAttributes)(tkwin, valueMask, attsPtr)
#define Tk_CharBbox(layout, index, xPtr, yPtr, widthPtr, heightPtr) \
	(tkStubTable->charBbox)(layout, index, xPtr, yPtr, widthPtr, \
	heightPtr)
#define Tk_ClearSelection(tkwin, selection) \
	(tkStubTable->clearSelection)(tkwin, selection)
#define Tk_ClipboardAppend(interp, tkwin, target, format, buffer) \
	(tkStubTable->clipboardAppend)(interp, tkwin, target, format, buffer)
#define Tk_ClipboardClear(interp, tkwin) \
	(tkStubTable->clipboardClear)(interp, tkwin)
#define Tk_ConfigureInfo(interp, tkwin, specs, widgRec, argvName, flags) \
	(tkStubTable->configureInfo)(interp, tkwin, specs, widgRec, \
	argvName, flags)
#define Tk_ConfigureValue(interp, tkwin, specs, widgRec, argvName, flags) \
	(tkStubTable->configureValue)(interp, tkwin, specs, widgRec, \
	argvName, flags)
#define Tk_ConfigureWidget(interp, tkwin, specs, argc, argv, widgRec, flags) \
	(tkStubTable->configureWidget)(interp, tkwin, specs, argc, argv, \
	widgRec, flags)
#define Tk_ConfigureWindow(tkwin, valueMask, valuePtr) \
	(tkStubTable->configureWindow)(tkwin, valueMask, valuePtr)
#define Tk_ComputeTextLayout(font, string, numChars, wrapLength, justify, \
	flags, widthPtr, heightPtr) \
	(tkStubTable->computeTextLayout)(font, string, numChars, wrapLength, \
	justify, flags, widthPtr, heightPtr)
#define Tk_CoordsToWindow(rootX, rootY, tkwin) \
	(tkStubTable->coordsToWindow)(rootX, rootY, tkwin)
#define Tk_CreateBinding(interp, bindingTable, object, eventString, command, \
	append) \
	(tkStubTable->createBinding)(interp, bindingTable, object, \
	eventString, command, append)
#define Tk_CreateBindingTable(interp) \
	(tkStubTable->createBindingTable)(interp)
#define Tk_CreateErrorHandler(display, errNum, request, minorCode, errorProc, \
	clientData) \
	(tkStubTable->createErrorHandler)(display, errNum, request, \
	minorCode, errorProc, clientData)
#define Tk_CreateEventHandler(token, mask, proc, clientData) \
	(tkStubTable->createEventHandler)(token, mask, proc, clientData)
#define Tk_CreateGenericHandler(proc, clientData) \
	(tkStubTable->createGenericHandler)(proc, clientData)
#define Tk_CreateImageType(typePtr) \
	(tkStubTable->createImageType)(typePtr)
#define Tk_CreateItemType(typePtr) \
	(tkStubTable->createItemType)(typePtr)
#define Tk_CreatePhotoImageFormat(formatPtr) \
	(tkStubTable->createPhotoImageFormat)(formatPtr)
#define Tk_CreateSelHandler(tkwin, selection, target, proc, clientData, \
	format) \
	(tkStubTable->createSelHandler)(tkwin, selection, target, proc, \
	clientData, format)
#define Tk_CreateWindow(interp, parent, name, screenName) \
	(tkStubTable->createWindow)(interp, parent, name, screenName)
#define Tk_CreateWindowFromPath(interp, tkwin, pathName, screenName) \
	(tkStubTable->createWindowFromPath)(interp, tkwin, pathName, \
	screenName)
#define Tk_DefineBitmap(interp, name, source, width, height) \
	(tkStubTable->defineBitmap)(interp, name, source, width, height)
#define Tk_DefineCursor(window, cursor) \
	(tkStubTable->defineCursor)(window, cursor)
#define Tk_DeleteAllBindings(bindingTable, object) \
	(tkStubTable->deleteAllBindings)(bindingTable, object)
#define Tk_DeleteBinding(interp, bindingTable, object, eventString) \
	(tkStubTable->deleteBinding)(interp, bindingTable, object, \
	eventString)
#define Tk_DeleteBindingTable(bindingTable) \
	(tkStubTable->deleteBindingTable)(bindingTable)
#define Tk_DeleteErrorHandler(handler) \
	(tkStubTable->deleteErrorHandler)(handler)
#define Tk_DeleteEventHandler(token, mask, proc, clientData) \
	(tkStubTable->deleteEventHandler)(token, mask, proc, clientData)
#define Tk_DeleteGenericHandler(proc, clientData) \
	(tkStubTable->deleteGenericHandler)(proc, clientData)
#define Tk_DeleteImage(interp, name) \
	(tkStubTable->deleteImage)(interp, name)
#define Tk_DeleteSelHandler(tkwin, selection, target) \
	(tkStubTable->deleteSelHandler)(tkwin, selection, target)
#define Tk_DestroyWindow(tkwin) \
	(tkStubTable->destroyWindow)(tkwin)
#define Tk_DisplayName(tkwin) \
	(tkStubTable->displayName)(tkwin)
#define Tk_DistanceToTextLayout(layout, x, y) \
	(tkStubTable->distanceToTextLayout)(layout, x, y)
#define Tk_Draw3DPolygon(tkwin, drawable, border, pointPtr, numPoints, \
	borderWidth, leftRelief) \
	(tkStubTable->draw3DPolygon)(tkwin, drawable, border, pointPtr, \
	numPoints, borderWidth, leftRelief)
#define Tk_Draw3DRectangle(tkwin, drawable, border, x, y, width, height, \
	borderWidth, relief) \
	(tkStubTable->draw3DRectangle)(tkwin, drawable, border, x, y, width, \
	height, borderWidth, relief)
#define Tk_DrawChars(display, drawable, gc, tkfont, source, numChars, x, y) \
	(tkStubTable->drawChars)(display, drawable, gc, tkfont, source, \
	numChars, x, y)
#define Tk_DrawFocusHighlight(tkwin, gc, width, drawable) \
	(tkStubTable->drawFocusHighlight)(tkwin, gc, width, drawable)
#define Tk_DrawTextLayout(display, drawable, gc, layout, x, y, firstChar, \
	lastChar) \
	(tkStubTable->drawTextLayout)(display, drawable, gc, layout, x, y, \
	firstChar, lastChar)
#define Tk_Fill3DPolygon(tkwin, drawable, border, pointPtr, numPoints, \
	borderWidth, leftRelief) \
	(tkStubTable->fill3DPolygon)(tkwin, drawable, border, pointPtr, \
	numPoints, borderWidth, leftRelief)
#define Tk_Fill3DRectangle(tkwin, drawable, border, x, y, width, height, \
	borderWidth, relief) \
	(tkStubTable->fill3DRectangle)(tkwin, drawable, border, x, y, width, \
	height, borderWidth, relief)
#define Tk_FindPhoto(interp, imageName) \
	(tkStubTable->findPhoto)(interp, imageName)
#define Tk_FontId(font) \
	(tkStubTable->fontId)(font)
#define Tk_Free3DBorder(border) \
	(tkStubTable->free3DBorder)(border)
#define Tk_FreeBitmap(display, bitmap) \
	(tkStubTable->freeBitmap)(display, bitmap)
#define Tk_FreeColor(colorPtr) \
	(tkStubTable->freeColor)(colorPtr)
#define Tk_FreeColormap(display, colormap) \
	(tkStubTable->freeColormap)(display, colormap)
#define Tk_FreeCursor(display, cursor) \
	(tkStubTable->freeCursor)(display, cursor)
#define Tk_FreeFont(font) \
	(tkStubTable->freeFont)(font)
#define Tk_FreeGC(display, gc) \
	(tkStubTable->freeGC)(display, gc)
#define Tk_FreeImage(image) \
	(tkStubTable->freeImage)(image)
#define Tk_FreeOptions(specs, widgRec, display, needFlags) \
	(tkStubTable->freeOptions)(specs, widgRec, display, needFlags)
#define Tk_FreePixmap(display, pixmap) \
	(tkStubTable->freePixmap)(display, pixmap)
#define Tk_FreeTextLayout(textLayout) \
	(tkStubTable->freeTextLayout)(textLayout)
#define Tk_GCForColor(colorPtr, drawable) \
	(tkStubTable->gCForColor)(colorPtr, drawable)
#define Tk_GeometryRequest(tkwin, reqWidth, reqHeight) \
	(tkStubTable->geometryRequest)(tkwin, reqWidth, reqHeight)
#define Tk_Get3DBorder(interp, tkwin, colorName) \
	(tkStubTable->get3DBorder)(interp, tkwin, colorName)
#define Tk_GetAllBindings(interp, bindingTable, object) \
	(tkStubTable->getAllBindings)(interp, bindingTable, object)
#define Tk_GetAnchor(interp, string, anchorPtr) \
	(tkStubTable->getAnchor)(interp, string, anchorPtr)
#define Tk_GetAtomName(tkwin, atom) \
	(tkStubTable->getAtomName)(tkwin, atom)
#define Tk_GetBinding(interp, bindingTable, object, eventString) \
	(tkStubTable->getBinding)(interp, bindingTable, object, eventString)
#define Tk_GetBitmap(interp, tkwin, string) \
	(tkStubTable->getBitmap)(interp, tkwin, string)
#define Tk_GetBitmapFromData(interp, tkwin, source, width, height) \
	(tkStubTable->getBitmapFromData)(interp, tkwin, source, width, \
	height)
#define Tk_GetCapStyle(interp, string, capPtr) \
	(tkStubTable->getCapStyle)(interp, string, capPtr)
#define Tk_GetColor(interp, tkwin, name) \
	(tkStubTable->getColor)(interp, tkwin, name)
#define Tk_GetColorByValue(tkwin, colorPtr) \
	(tkStubTable->getColorByValue)(tkwin, colorPtr)
#define Tk_GetColormap(interp, tkwin, string) \
	(tkStubTable->getColormap)(interp, tkwin, string)
#define Tk_GetCursor(interp, tkwin, string) \
	(tkStubTable->getCursor)(interp, tkwin, string)
#define Tk_GetCursorFromData(interp, tkwin, source, mask, width, height, xHot, \
	yHot, fg, bg) \
	(tkStubTable->getCursorFromData)(interp, tkwin, source, mask, width, \
	height, xHot, yHot, fg, bg)
#define Tk_GetFont(interp, tkwin, string) \
	(tkStubTable->getFont)(interp, tkwin, string)
#define Tk_GetFontFromObj(interp, tkwin, objPtr) \
	(tkStubTable->getFontFromObj)(interp, tkwin, objPtr)
#define Tk_GetFontMetrics(font, fmPtr) \
	(tkStubTable->getFontMetrics)(font, fmPtr)
#define Tk_GetGC(tkwin, valueMask, valuePtr) \
	(tkStubTable->getGC)(tkwin, valueMask, valuePtr)
#define Tk_GetImage(interp, tkwin, name, changeProc, clientData) \
	(tkStubTable->getImage)(interp, tkwin, name, changeProc, clientData)
#define Tk_GetImageMasterData(interp, name, typePtrPtr) \
	(tkStubTable->getImageMasterData)(interp, name, typePtrPtr)
#define Tk_GetItemTypes() \
	(tkStubTable->getItemTypes)()
#define Tk_GetJoinStyle(interp, string, joinPtr) \
	(tkStubTable->getJoinStyle)(interp, string, joinPtr)
#define Tk_GetJustify(interp, string, justifyPtr) \
	(tkStubTable->getJustify)(interp, string, justifyPtr)
#define Tk_GetNumMainWindows() \
	(tkStubTable->getNumMainWindows)()
#define Tk_GetOption(tkwin, name, className) \
	(tkStubTable->getOption)(tkwin, name, className)
#define Tk_GetPixels(interp, tkwin, string, intPtr) \
	(tkStubTable->getPixels)(interp, tkwin, string, intPtr)
#define Tk_GetPixmap(display, d, width, height, depth) \
	(tkStubTable->getPixmap)(display, d, width, height, depth)
#define Tk_GetRelief(interp, name, reliefPtr) \
	(tkStubTable->getRelief)(interp, name, reliefPtr)
#define Tk_GetRootCoords(tkwin, xPtr, yPtr) \
	(tkStubTable->getRootCoords)(tkwin, xPtr, yPtr)
#define Tk_GetScrollInfo(interp, argc, argv, dblPtr, intPtr) \
	(tkStubTable->getScrollInfo)(interp, argc, argv, dblPtr, intPtr)
#define Tk_GetScreenMM(interp, tkwin, string, doublePtr) \
	(tkStubTable->getScreenMM)(interp, tkwin, string, doublePtr)
#define Tk_GetSelection(interp, tkwin, selection, target, proc, clientData) \
	(tkStubTable->getSelection)(interp, tkwin, selection, target, proc, \
	clientData)
#define Tk_GetUid(string) \
	(tkStubTable->getUid)(string)
#define Tk_GetVisual(interp, tkwin, string, depthPtr, colormapPtr) \
	(tkStubTable->getVisual)(interp, tkwin, string, depthPtr, \
	colormapPtr)
#define Tk_GetVRootGeometry(tkwin, xPtr, yPtr, widthPtr, heightPtr) \
	(tkStubTable->getVRootGeometry)(tkwin, xPtr, yPtr, widthPtr, \
	heightPtr)
#define Tk_Grab(interp, tkwin, grabGlobal) \
	(tkStubTable->grab)(interp, tkwin, grabGlobal)
#define Tk_HandleEvent(eventPtr) \
	(tkStubTable->handleEvent)(eventPtr)
#define Tk_IdToWindow(display, window) \
	(tkStubTable->idToWindow)(display, window)
#define Tk_ImageChanged(master, x, y, width, height, imageWidth, imageHeight) \
	(tkStubTable->imageChanged)(master, x, y, width, height, imageWidth, \
	imageHeight)
#define Tk_Init(interp) \
	(tkStubTable->init)(interp)
#define Tk_InternAtom(tkwin, name) \
	(tkStubTable->internAtom)(tkwin, name)
#define Tk_IntersectTextLayout(layout, x, y, width, height) \
	(tkStubTable->intersectTextLayout)(layout, x, y, width, height)
#define TkMain(argc, argv, appInitProc, interp) \
	(tkStubTable->main_)(argc, argv, appInitProc, interp)
#define Tk_MaintainGeometry(slave, master, x, y, width, height) \
	(tkStubTable->maintainGeometry)(slave, master, x, y, width, height)
#define Tk_MainWindow(interp) \
	(tkStubTable->mainWindow)(interp)
#define Tk_MakeWindowExist(tkwin) \
	(tkStubTable->makeWindowExist)(tkwin)
#define Tk_ManageGeometry(tkwin, mgrPtr, clientData) \
	(tkStubTable->manageGeometry)(tkwin, mgrPtr, clientData)
#define Tk_MapWindow(tkwin) \
	(tkStubTable->mapWindow)(tkwin)
#define Tk_MeasureChars(tkfont, source, maxChars, maxPixels, flags, \
	lengthPtr) \
	(tkStubTable->measureChars)(tkfont, source, maxChars, maxPixels, \
	flags, lengthPtr)
#define Tk_MoveResizeWindow(tkwin, x, y, width, height) \
	(tkStubTable->moveResizeWindow)(tkwin, x, y, width, height)
#define Tk_MoveWindow(tkwin, x, y) \
	(tkStubTable->moveWindow)(tkwin, x, y)
#define Tk_MoveToplevelWindow(tkwin, x, y) \
	(tkStubTable->moveToplevelWindow)(tkwin, x, y)
#define Tk_NameOf3DBorder(border) \
	(tkStubTable->nameOf3DBorder)(border)
#define Tk_NameOfAnchor(anchor) \
	(tkStubTable->nameOfAnchor)(anchor)
#define Tk_NameOfBitmap(display, bitmap) \
	(tkStubTable->nameOfBitmap)(display, bitmap)
#define Tk_NameOfCapStyle(cap) \
	(tkStubTable->nameOfCapStyle)(cap)
#define Tk_NameOfColor(colorPtr) \
	(tkStubTable->nameOfColor)(colorPtr)
#define Tk_NameOfCursor(display, cursor) \
	(tkStubTable->nameOfCursor)(display, cursor)
#define Tk_NameOfFont(font) \
	(tkStubTable->nameOfFont)(font)
#define Tk_NameOfImage(imageMaster) \
	(tkStubTable->nameOfImage)(imageMaster)
#define Tk_NameOfJoinStyle(join) \
	(tkStubTable->nameOfJoinStyle)(join)
#define Tk_NameOfJustify(justify) \
	(tkStubTable->nameOfJustify)(justify)
#define Tk_NameOfRelief(relief) \
	(tkStubTable->nameOfRelief)(relief)
#define Tk_NameToWindow(interp, pathName, tkwin) \
	(tkStubTable->nameToWindow)(interp, pathName, tkwin)
#define Tk_OwnSelection(tkwin, selection, proc, clientData) \
	(tkStubTable->ownSelection)(tkwin, selection, proc, clientData)
#define Tk_ParseArgv(interp, tkwin, argcPtr, argv, argTable, flags) \
	(tkStubTable->parseArgv)(interp, tkwin, argcPtr, argv, argTable, \
	flags)
#define Tk_PhotoPutBlock(handle, blockPtr, x, y, width, height) \
	(tkStubTable->photoPutBlock)(handle, blockPtr, x, y, width, height)
#define Tk_PhotoPutZoomedBlock(handle, blockPtr, x, y, width, height, zoomX, \
	zoomY, subsampleX, subsampleY) \
	(tkStubTable->photoPutZoomedBlock)(handle, blockPtr, x, y, width, \
	height, zoomX, zoomY, subsampleX, subsampleY)
#define Tk_PhotoGetImage(handle, blockPtr) \
	(tkStubTable->photoGetImage)(handle, blockPtr)
#define Tk_PhotoBlank(handle) \
	(tkStubTable->photoBlank)(handle)
#define Tk_PhotoExpand(handle, width, height) \
	(tkStubTable->photoExpand)(handle, width, height)
#define Tk_PhotoGetSize(handle, widthPtr, heightPtr) \
	(tkStubTable->photoGetSize)(handle, widthPtr, heightPtr)
#define Tk_PhotoSetSize(handle, width, height) \
	(tkStubTable->photoSetSize)(handle, width, height)
#define Tk_PointToChar(layout, x, y) \
	(tkStubTable->pointToChar)(layout, x, y)
#define Tk_PostscriptFontName(tkfont, dsPtr) \
	(tkStubTable->postscriptFontName)(tkfont, dsPtr)
#define Tk_PreserveColormap(display, colormap) \
	(tkStubTable->preserveColormap)(display, colormap)
#define Tk_QueueWindowEvent(eventPtr, position) \
	(tkStubTable->queueWindowEvent)(eventPtr, position)
#define Tk_RedrawImage(image, imageX, imageY, width, height, drawable, \
	drawableX, drawableY) \
	(tkStubTable->redrawImage)(image, imageX, imageY, width, height, \
	drawable, drawableX, drawableY)
#define Tk_ResizeWindow(tkwin, width, height) \
	(tkStubTable->resizeWindow)(tkwin, width, height)
#define Tk_RestackWindow(tkwin, aboveBelow, other) \
	(tkStubTable->restackWindow)(tkwin, aboveBelow, other)
#define Tk_RestrictEvents(proc, arg, prevArgPtr) \
	(tkStubTable->restrictEvents)(proc, arg, prevArgPtr)
#define Tk_SafeInit(interp) \
	(tkStubTable->safeInit)(interp)
#define Tk_SetAppName(tkwin, name) \
	(tkStubTable->setAppName)(tkwin, name)
#define Tk_SetBackgroundFromBorder(tkwin, border) \
	(tkStubTable->setBackgroundFromBorder)(tkwin, border)
#define Tk_SetClass(tkwin, className) \
	(tkStubTable->setClass)(tkwin, className)
#define Tk_SetGrid(tkwin, reqWidth, reqHeight, gridWidth, gridHeight) \
	(tkStubTable->setGrid)(tkwin, reqWidth, reqHeight, gridWidth, \
	gridHeight)
#define Tk_SetInternalBorder(tkwin, width) \
	(tkStubTable->setInternalBorder)(tkwin, width)
#define Tk_SetWindowBackground(tkwin, pixel) \
	(tkStubTable->setWindowBackground)(tkwin, pixel)
#define Tk_SetWindowBackgroundPixmap(tkwin, pixmap) \
	(tkStubTable->setWindowBackgroundPixmap)(tkwin, pixmap)
#define Tk_SetWindowBorder(tkwin, pixel) \
	(tkStubTable->setWindowBorder)(tkwin, pixel)
#define Tk_SetWindowBorderWidth(tkwin, width) \
	(tkStubTable->setWindowBorderWidth)(tkwin, width)
#define Tk_SetWindowBorderPixmap(tkwin, pixmap) \
	(tkStubTable->setWindowBorderPixmap)(tkwin, pixmap)
#define Tk_SetWindowColormap(tkwin, colormap) \
	(tkStubTable->setWindowColormap)(tkwin, colormap)
#define Tk_SetWindowVisual(tkwin, visual, depth, colormap) \
	(tkStubTable->setWindowVisual)(tkwin, visual, depth, colormap)
#define Tk_SizeOfBitmap(display, bitmap, widthPtr, heightPtr) \
	(tkStubTable->sizeOfBitmap)(display, bitmap, widthPtr, heightPtr)
#define Tk_SizeOfImage(image, widthPtr, heightPtr) \
	(tkStubTable->sizeOfImage)(image, widthPtr, heightPtr)
#define Tk_StrictMotif(tkwin) \
	(tkStubTable->strictMotif)(tkwin)
#define Tk_TextLayoutToPostscript(interp, layout) \
	(tkStubTable->textLayoutToPostscript)(interp, layout)
#define Tk_TextWidth(font, string, numChars) \
	(tkStubTable->textWidth)(font, string, numChars)
#define Tk_UndefineCursor(window) \
	(tkStubTable->undefineCursor)(window)
#define Tk_UnderlineChars(display, drawable, gc, tkfont, source, x, y, \
	firstChar, lastChar) \
	(tkStubTable->underlineChars)(display, drawable, gc, tkfont, source, \
	x, y, firstChar, lastChar)
#define Tk_UnderlineTextLayout(display, drawable, gc, layout, x, y, \
	underline) \
	(tkStubTable->underlineTextLayout)(display, drawable, gc, layout, x, \
	y, underline)
#define Tk_Ungrab(tkwin) \
	(tkStubTable->ungrab)(tkwin)
#define Tk_UnmaintainGeometry(slave, master) \
	(tkStubTable->unmaintainGeometry)(slave, master)
#define Tk_UnmapWindow(tkwin) \
	(tkStubTable->unmapWindow)(tkwin)
#define Tk_UnsetGrid(tkwin) \
	(tkStubTable->unsetGrid)(tkwin)

#define Tk_Required(interp, version, exact) \
		((Tcl_FakeInterp*) (interp))->stub->pkgRequire2(interp, \
		"Tk", version, exact, (ClientData *) &tkStubTable)

#endif /* TK_USE_STUBS */

#endif /* RESOURCE_INCLUDED */

#undef TCL_STORAGE_CLASS
#define TCL_STORAGE_CLASS DLLIMPORT

#endif /* _TK */
